#include "dpre.h"

#include <rcsc/common/server_param.h>

#include <rcsc/common/logger.h>

#include <fcportugal_player.h>

#include "planning.h"
#include <strategy.h>
#include "fcportugal/strategy_2d.h"

#include <sstream>
#include <iostream>


using namespace rcsc;

namespace fcportugal {

    DPRE& DPRE::instance()
    {
        static DPRE i;

        return i;
    }

    DPRE::DPRE()
        : soccer::strategy::DPRE()
    {
        for ( unsigned int i = 1; i <= ServerParam::i().maxPlayer() ; i++ )
        {
            M_positionings[i] = i;
        }
    }
	
	DPRE::~DPRE()
	{
	
	}

    void DPRE::setAgent( const FCPortugalPlayer* agent )
    {
        M_agent = agent;
    }

    /*!
    \brief Verify if player should be considered for DPRE
    \return True if the player is to be considered or false otherwise
     */
    /*bool DPRE::considerPlayer( unsigned int number ) const
    {
        const AbstractPlayerObject* player = M_agent->world().ourPlayer(number );
        if ( player == NULL )
        {
            dlog.addText( Logger::POSITIONING,
                         __FILE__" Disregard teammate %d because his reference was not found", number );
            return false;
        }

        // DO NOT DPRE with Goalie
        if ( player->goalie() )
        {
            return false;
        }

        // Disregard role exchanges between with any players that are participating in a plan
        if ( sagent->planner()->participates( PlayerID( TA_OUR, number  ) ))
        {
            dlog.addText( Logger::POSITIONING,
                         __FILE__" Disregard teammate %d because he is participating in a plan", number );
            return false;
        }

        return true;
    } */

    // Dynamic positioning and role exchange
    void DPRE::update( )
    {
        if ( M_agent->world().gameMode().isTeamsSetPlay(LEFT) ||
             M_agent->world().gameMode().isTeamsSetPlay(RIGHT) ||
             M_agent->world().gameMode().isPenaltyKickMode() ||
             (M_agent->world().gameMode().type() == rcsc::GameMode::GoalieCatch_))
        {
            dlog.addText( Logger::POSITIONING,
                         __FILE__  " Reset DPRE playmode %d",
                          M_agent->world().gameMode().getServerPlayMode());
        }

        if ( M_agent->world().gameMode().type() != rcsc::GameMode::KickOff_ &&
             M_agent->world().gameMode().type() != rcsc::GameMode::BeforeKickOff)
        {
            return;
        }

        // Determine eligible players

        // Calculate weights for players exchanges
        return soccer::strategy::DPRE::update( );
    }


    /*!
      \brief Retrieve the list of positionings that were changed in the last analysis
      */
    void DPRE::positioningsChanged(Positionings changed_positionings)
    {

    }

    /*!
    \brief Determine players eligible for DPRE
    \return True if the player is to be considered or false otherwise
     */
    const std::vector<unsigned int>& DPRE::eligiblePlayers( )
    {
        M_eligible_players.clear();
        stringstream ss;
        ss << "EligiblePlayersForDPRE: ";
        // TODO All players are eligible except set-play participants
        for ( unsigned int i = 1; i < 12 ; i++)
        {
            const AbstractPlayerObject* player = M_agent->world().ourPlayer( i);
            if ( player == NULL )
            {
                ss << " (No p" << i << ")";
                continue;
            }

            if ( player->goalie() )
            {
                ss << " (Excluding goalie " << i << ")";
                continue;
                // This should be done externally based on strategy and setplay
                // eligible_players.erase( *itp );
            }
            // If player participates in a plan, also remove him
            ss << player->unum() << " ";
            M_eligible_players.push_back( player->unum() );
        }
        dlog.addText( Logger::POSITIONING, __FILE__ " DPRE: %s", ss.str().c_str() );
        dlog.flush();
        return M_eligible_players;
    }

    /*!
      \brief Value of exchanging the positionings of 2 players
      \return True if it is useful to exchange player positionings
      */
    double DPRE::exchangeValue(unsigned int number1,
                               unsigned int number2) const
    {
        // Get players current position
        const Vector2D& p1_pos = M_agent->world().ourPlayer( number1 )->pos();
        const Vector2D& p2_pos = M_agent->world().ourPlayer( number2 )->pos();

        // Get players strategic position
        const rcsc::Vector2D& p1_spos = fcportugal::strategy::Strategy2D::instance().position(number1);
        const rcsc::Vector2D& p2_spos = fcportugal::strategy::Strategy2D::instance().position(number2);

        // Calculate p1 distances to his strategic position and of p2
        double p1_dist_to_p1_spos = p1_pos.dist(p1_spos);
        double p1_dist_to_p2_spos = p1_pos.dist(p2_spos);

        // Calculate p2 distances to his strategic position and of p2
        double p2_dist_to_p1_spos = p2_pos.dist(p1_spos);
        double p2_dist_to_p2_spos = p2_pos.dist(p2_spos);

        /* Utility metric
         * The sum of the distance from each players to the others strategic positions is lower
         * than the sum of the distances to their own strategic positions
         */
        double dist_to_other_spos = p2_dist_to_p1_spos + p1_dist_to_p2_spos;
        double dist_to_self_spos = p1_dist_to_p1_spos + p2_dist_to_p2_spos;

        return dist_to_self_spos - dist_to_other_spos;
        if ( dist_to_other_spos < dist_to_self_spos )
        {
            dlog.addText( Logger::POSITIONING,
                         __FILE__ " DPRE (%d,%d) p1pos(%.2f,%.2f) p2pos(%.2f,%.2f)"
                                  " dist_self_spos(%.2f,%.2f)"
                                  " dist_other_spos(%.2f,%.2f)"
                                  " %.2f < %.2f",
                         number1, number2,
                         p1_pos.x, p1_pos.y,
                         p2_pos.x, p2_pos.y,
                         p1_dist_to_p1_spos, p2_dist_to_p2_spos,
                         p1_dist_to_p2_spos, p2_dist_to_p1_spos,
                         dist_to_other_spos, dist_to_self_spos);
            dlog.flush();
        }
    }
}
