#include <fstream>
#include <iostream>
#include <sstream>
#include "identifyopponent.h"
#include <boost/algorithm/string.hpp>
#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_assign_actor.hpp>

using namespace BOOST_SPIRIT_CLASSIC_NS;
using namespace boost;
using namespace std;

TeamDesc::TeamDesc(unsigned short teamNumber, vector<string> names, 
		   unsigned char teamQual, unsigned char defenseQual, 
		   unsigned char attackQual, unsigned char playQual, 
		   unsigned char goalieQual)
  :teamNumber_(teamNumber),names_(names),teamQual_(teamQual),
   defenseQual_(defenseQual),attackQual_(attackQual),
   playQual_(playQual),goalieQual_(goalieQual)
{}

TeamDesc* TeamDesc::parse(string in, string& out){
  unsigned int teamN, teamQual, defenseQual, 
    attackQual, playQual, goalieQual;
  //parse qualities  
  rule<> rule_p
    = *space_p 
    >> int_p[assign_a(teamN)] >> *space_p 
    >> int_p[assign_a(teamQual)] >> *space_p 
    >> int_p[assign_a(defenseQual)] >> *space_p 
    >> int_p[assign_a(attackQual)] >> *space_p 
    >> int_p[assign_a(playQual)] >> *space_p  
    >> int_p[assign_a(goalieQual)] >> *space_p 
    >> *space_p;
  
  parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in.c_str(),rule_p,
						       nothing_p);
  
  if(result.hit){
    in=in.substr(result.length);
    // parse names, arbitrary amount
    vector<string> names;
    
    while(in.size()>0){
      //find space
      size_t space_pos=in.find(' ');
      if(space_pos==string::npos){
	names.push_back(in);
	in="";
      }
      else{
	names.push_back(in.substr(0,space_pos));
	in=in.substr(space_pos+1);
      }
      
      if(in.find_first_not_of(" \t")!=string::npos)
	in=in.substr(in.find_first_not_of(" \t"));
    }

    return new TeamDesc((unsigned short)teamN,names,(unsigned char)teamQual,
			(unsigned char)defenseQual,(unsigned char)attackQual,
			(unsigned char)playQual,(unsigned char)goalieQual);
  }
  return NULL;
}


string TeamDesc::toString() const{
  ostringstream out;

  out<<"Team:"<<teamNumber()<<" TQ:"<<(unsigned int)teamQual()
     <<" DQ:"<<(unsigned int)defenseQual()
     <<" AQ:"<<(unsigned int)attackQual()<<" PQ:"<<(unsigned int)playQual()
     <<" GQ:"<<(unsigned int)goalieQual()<< "names:";

  for(unsigned int i=0; i!= names().size();i++)
    out<<names()[i]<<" ";

  return out.str();
}

TeamDescContainer* TeamDescContainer::parseFromFile(string fileName){
  string line;
  ifstream textstream(fileName.c_str());

  if(!textstream.is_open()){
    cerr<<"\n Could not open "<<fileName<<"!\n";
    return NULL;
  }

  map<unsigned short,TeamDesc*> teams;
  
  while (getline(textstream, line)){
    if(!line.length()==0 || line[0]=='%'){
      string rest;
      TeamDesc* team=TeamDesc::parse(line,rest);
      
      if(team) teams.insert(make_pair(team->teamNumber(),
				      team));
    }
  }
  
  textstream.close();
  
  return new TeamDescContainer(teams);
  
}

const TeamDesc* TeamDescContainer::team(unsigned short nbr) const{
  return (teams_.find(nbr)==teams_.end()?NULL:teams_.find(nbr)->second);}

TeamDesc* TeamDescContainer::identifyOpponent(string name) const{
  //filter unnecessary information from name: '09','2D',etc
  ierase_all(name,"2D");
  ierase_all(name,"2009");
  ierase_all(name,"09");
  ierase_all(name,"2010");
  ierase_all(name,"10");
  ierase_all(name,"_");

  double bestFit=0;
  TeamDesc* opp=NULL;

  for(map<unsigned short,TeamDesc*>::const_iterator i=teams_.begin(); 
      i!= teams_.end(); i++)
    for(unsigned int j=0; j!= i->second->names().size(); j++)
      if(!ifind_first(i->second->names()[j],name).empty()
	 ||!ifind_first(name,i->second->names()[j]).empty()){
	bestFit=i->second->names()[j].length()/name.length();
	opp=i->second;
      }

  return opp;
}
