// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifndef FCPORTUGAL_PLAYER_H
#define FCPORTUGAL_PLAYER_H

#include <chain_action/action_generator.h>
#include <chain_action/field_evaluator.h>

#include <communication/communication.h>

#include <rcsc/player/player_agent.h>

#include <vector>

// #include <fcportugal/planning.h>
// #include <fcportugal/flux_manager.h>

#include <setplay/setplay_world_model.h>
#include "world_model.h"

#include <setplay/runtime/setplay_interpreter.h>

namespace setplay {
    class IConditionEvaluator;
    class IActionExecutor;
    class ISetplaySelector;
    class IPlayerSelector;
    class SetplayInterpreter;
    class SetplayCommunication;
    class SetplayDatabase;
}

namespace fcportugal
{
    namespace strategy
    {
        class Strategy2D;
    }

}

class FCPortugalPlayer
    : public rcsc::PlayerAgent {
private:
    fcportugal::SetplayWorldModel* M_setplay_world_model;
    soccer::WorldModel* M_world_model;
    // Should this be instead be a member of Communication?
    // Agent-specific implementations
    setplay::IConditionEvaluator* M_setplay_condition_evaluator;
    setplay::IActionExecutor* M_setplay_action_executor;
    setplay::ISetplaySelector* M_setplay_selector;
    setplay::IPlayerSelector* M_player_selector;
    setplay::SetplayInterpreter* M_setplay_interpreter;
    setplay::SetplayCommunication* M_setplay_communication;
    setplay::SetplayDatabase* M_setplay_database;

    Communication::Ptr M_communication;

    FieldEvaluator::ConstPtr M_field_evaluator;
    ActionGenerator::ConstPtr M_action_generator;

public:
    FCPortugalPlayer();

    virtual
    ~FCPortugalPlayer();

protected:
    /*!
      You can override this method.
      But you must call PlayerAgent::initImpl() in this method.
    */
    virtual
    bool initImpl( rcsc::CmdLineParser & cmd_parser );

    //! main decision
    virtual
    void actionImpl();

    //! communication decision
    virtual
    void communicationImpl();

    virtual
    void handleActionStart();
    virtual
    void handleActionEnd();

    virtual
    void handleServerParam();
    virtual
    void handlePlayerParam();
    virtual
    void handlePlayerType();

    virtual
    FieldEvaluator::ConstPtr createFieldEvaluator() const;

    virtual
    ActionGenerator::ConstPtr createActionGenerator() const;

private:
    void loadSetplays();

    bool doPreprocess();
    bool doShoot();
    bool doForceKick();
    bool doHeardPassReceive();

public:
    virtual
    FieldEvaluator::ConstPtr getFieldEvaluator() const;

    const setplay::SetplayInterpreter& setplayInterpreter() const;

};

#endif
