#ifndef SETPLAY_CONDITION_EVALUATOR_H
#define SETPLAY_CONDITION_EVALUATOR_H

#include <rcsc/player/world_model.h>
#include <setplay/runtime/icondition_evaluator.h>

namespace fcportugal {
    class SetplayWorldModel;
}

class SetplayConditionEvaluator
        : public setplay::IConditionEvaluator
{
public:
    SetplayConditionEvaluator(const fcportugal::SetplayWorldModel* world);

    /// IConditionEvaluator interface implementation
    bool playerAtRegion( const setplay::PlayerAffiliation& player,
                         const rcsc::Region2D& region );
    bool playerAtPosition( const setplay::PlayerAffiliation& player,
                           const rcsc::Vector2D& position );

    bool ballAtRegion( const rcsc::Region2D& region );
    bool ballAtPosition( const rcsc::Vector2D& position );

    bool canPassToPlayer( const setplay::PlayerAffiliation& player1,
                          const setplay::PlayerAffiliation& player2);
    bool canPassToRegion( const setplay::PlayerAffiliation& player,
                          const rcsc::Region2D& region );
    bool canPassToPosition( const setplay::PlayerAffiliation& player,
                            const rcsc::Vector2D& position );

    bool isOffside( const setplay::PlayerAffiliation& player );

    bool canInterceptBall( const setplay::PlayerAffiliation& player );

    bool isBallKickable( const setplay::PlayerAffiliation& player );
    bool ownsBall( const setplay::PlayerAffiliation &player);

    bool isTeamInPossession ( const setplay::TeamAffiliation& affiliation);

    bool isGameSituation( const setplay::GameSituation& game_situation);
private:
    static const double S_position_reached_distance_threshold;

    const fcportugal::SetplayWorldModel* M_setplay_world;
};

#endif // SETPLAY_CONDITION_EVALUATOR_H
