#ifndef SETPLAY_MESSAGE_BUILDER_H
#define SETPLAY_MESSAGE_BUILDER_H

#include <iostream>
#include <rcsc/player/say_message_builder.h>
#include <setplay/communication/messages.h>

/// Helper functions
class SaySetplayMessage
        : public rcsc::SayMessage
{
protected:
    std::ostream & printBase( const setplay::IMessage& msg, std::ostream & os ) const;
    std::ostream & printArguments( const setplay::IMessage& msg, std::ostream & os ) const;

    void encodeSetplayMessageBase(const setplay::IMessage& msg, std::string& to) const;
    void encodeSetplayMessageArguments( const setplay::IMessage& msg, std::string& to ) const;

    /*!
      \brief Get the size in bytes occupied a general by the set-play message
      */
    unsigned int lengthSetplayMessageBase(const setplay::IMessage& ) const;
    unsigned int lengthSetplayMessageArguments(const setplay::IMessage& msg) const;
};

class SaySetplayStartMessage
        : public SaySetplayMessage
{
public:
    SaySetplayStartMessage( const setplay::SetplayStartMessage* msg );
    ~SaySetplayStartMessage();

    // Implement SayMessage functions
    char header() const;
    int length() const;

    bool toStr( std::string & to ) const;
    std::ostream & printDebug( std::ostream & os ) const;

private:
    const setplay::SetplayStartMessage* M_msg;
};

class SayStepEnterMessage
        : public SaySetplayMessage
{
public:
    SayStepEnterMessage( const setplay::StepEnterMessage* msg );
    ~SayStepEnterMessage();

    // Implement SayMessage functions
    char header() const;
    int length() const;

    bool toStr( std::string & to ) const;
    std::ostream & printDebug( std::ostream & os ) const;

private:
    const setplay::StepEnterMessage* M_msg;
};

class SayStepTransitionMessage
        : public SaySetplayMessage
{
public:
    SayStepTransitionMessage( const setplay::StepTransitionMessage* msg );
    ~SayStepTransitionMessage();

    // Implement SayMessage functions
    char header() const;
    int length() const;

    bool toStr( std::string & to ) const;
    std::ostream & printDebug( std::ostream & os ) const;

private:
    void encodeSteps( std::string& to ) const;

    const setplay::StepTransitionMessage* M_msg;
};


class SaySetplayAbortMessage
        : public SaySetplayMessage
{
public:
    SaySetplayAbortMessage( const setplay::SetplayAbortMessage *msg );

    // Implement SayMessage functions
    char header() const;
    int length() const;

    bool toStr( std::string & to ) const;
    std::ostream & printDebug( std::ostream & os ) const;
private:
    const setplay::SetplayAbortMessage* M_msg;
};

class SaySetplayFinishMessage
        : public SaySetplayMessage
{
public:
    SaySetplayFinishMessage( const setplay::SetplayFinishMessage* msg );

    // Implement SayMessage functions
    char header() const;
    int length() const;

    bool toStr( std::string & to ) const;
    std::ostream & printDebug( std::ostream & os ) const;

private:
    const setplay::SetplayFinishMessage* M_msg;
};

/* NOT USED FOR NOW
class ActionDoneMessage
        : public rcsc::SayMessage,
          public fcportugal::setplay::ActionDoneMessage
{
public:
    ActionDoneMessage();

    // Implement SayMessage functions
    char header() const;
    int length() const;

    bool toStr( std::string & to ) const;
    std::ostream & printDebug( std::ostream & os ) const;
};*/

#endif // SETPLAY_MESSAGE_BUILDER_H
