#include "act_pass.h"

#include <setplay/model/ivisitor.h>

namespace setplay {


    ActPass::ActPass( ILocation2D* location, BallMoveType type)
     : M_location( location )
     , M_ball_move_type(type)
    {

    }

    ActPass::ActPass( const ActPass& other)
    {
        M_location = other.M_location != NULL ? other.M_location->clone() : NULL;
        M_ball_move_type = other.M_ball_move_type;
    }

    ActPass::~ActPass()
    {
        if ( M_location != NULL ) delete M_location;
    }

    TokenType ActPass::type() const
    {
        return TT_ACTION_PASS;
    }

    IAction::ActionType ActPass::actionType() const
    {
        return AT_PASS;
    }

    ILocation2D* ActPass::location() const
    {
        return M_location;
    }

    ActPass::BallMoveType ActPass::ballMoveType() const
    {
        return M_ball_move_type;
    }


    ActPass& ActPass::setLocation(ILocation2D *location)
    {
        if ( M_location != NULL ) { delete M_location; }
        M_location = location;
        return *this;
    }

    ActPass& ActPass::setBallMoveType(BallMoveType ball_move_type)
    {
        M_ball_move_type = ball_move_type;
        return *this;
    }

    ActPass& ActPass::operator*()
    {
        return *this;
    }

    ActPass* ActPass::clone() const
    {
        return new ActPass(*this);
    }

    ActPass& ActPass::operator=(const ActPass& other)
    {
      if (this != &other)
      {
          setLocation(location()->clone());
          M_ball_move_type = other.M_ball_move_type;
      }

      return *this;
    }


    void ActPass::visit(IVisitor& visitor)
    {
        visitor.visit( *this );
    }

}
