// -*-c++-*-

/***************************************************************************
                                IAction.h
                       Abstract base class for CLang actions
                             -------------------
    begin                : 28-MAY-2002
    copyright            : (C) 2006 Luis Mota
    email                : luis.mota@iscte.pt
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU LGPL as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option) any  *
 *   later version.                                                        *
 *                                                                         *
 ***************************************************************************/
#ifndef SP_ACTION_H
#define SP_ACTION_H

#include <cassert>
#include <setplay/model/object.h>

using namespace std;

namespace setplay {
	class ICondition;

    // An action is executable by an executor
    class IAction
        : public Object {
    public:
      virtual
      ~IAction()
        { }

        // Action Types
        typedef enum {
            AT_POS,
            AT_FORWARD,
            AT_PASS,
            AT_MARK,
            AT_MARK_LINE_PLAYER,
            AT_MARK_LINE_REG,
            AT_DRIBBLE,
            AT_OFFSIDE_LINE,
            AT_CLEAR,
            AT_SHOOT,
            AT_HOLD,
            AT_INTERCEPT,
            AT_INTERCEPT_PLAYER,
            AT_TACKLE,
            AT_SEQ,
            //Actions introduced in 2009:
            AT_STOP,
            AT_ATTENTION_TO_REG,
            AT_LOOK_AT,
            AT_MARK_GOAL,
            // Actions introduced when applying to the middle-size league
            AT_RECEIVE_BALL,
            //Actions introduced in 2010
            AT_MOVE_TO_OFFSIDE_LINE,
            //Actions introduced in 2012 by FA
            AT_FOLLOW
        } ActionType;


      virtual IAction& operator*() = 0;
      virtual ActionType actionType() const = 0;
      virtual IAction* clone() const = 0;

    }; // end IAction

    typedef vector<IAction*> Actions;
}

#endif
