#include "cond_can_pass.h"

namespace setplay {

CondCanPass::CondCanPass(IPlayer* player, ILocation2D* location)
    : M_player( player )
    , M_location( location )
{

}

CondCanPass::CondCanPass(const CondCanPass& other)
{
    M_player = other.M_player != NULL ? other.M_player->clone() : NULL;
    M_location = other.M_location != NULL ? other.M_location->clone() : NULL;
}

CondCanPass::~CondCanPass()
{
    if ( M_player != NULL ) { delete M_player; }
    if ( M_location != NULL ) { delete M_location; }
}

TokenType CondCanPass::type() const
{
    return TT_CONDITION_CAN_PASS;
}

IPlayer* CondCanPass::player( ) const
{
    return M_player;
}

ILocation2D* CondCanPass::location( ) const
{
    return M_location;
}

CondCanPass& CondCanPass::setPlayer(IPlayer* player)
{
    if ( M_player != NULL ) { delete M_player; }
    M_player = player;
    return *this;
}

CondCanPass& CondCanPass::setLocation(ILocation2D* location)
{
    if ( M_location != NULL ) { delete M_location; }
    M_location = location;
    return *this;
}

CondCanPass* CondCanPass::clone() const
{
    return new CondCanPass( *this );
}

CondCanPass& CondCanPass::operator*()
{
    return *this;
}

void CondCanPass::visit(IVisitor& visitor)
{
    visitor.visit( *this );
}

}
