// -*-c++-*-

/***************************************************************************
                                   cond_equals.h  
                               Clang conditions
                             -------------------
    begin                : 28-NOV-2006
    copyright            : (C) 2006 by Luis Mota
    email                : luis.mota@iscte.pt
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU LGPL as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option) any  *
 *   later version.                                                       *
 *                                                                         *
 ***************************************************************************/

#ifndef COND_EQUALS_H
#define COND_EQUALS_H

#include <setplay/model/conditions/icondition.h>

namespace setplay {

        class IPlayer;
	class IConditionEvaluator;

    // Class to model operations to check for equality between 2 objects of the same type
    template< typename A, typename B >
    class CondEquals: public ICondition {
    public:
      // Pointer to a function that takes 2 objects and returns a boolean
      // typedef bool (*EqualityFunc)(OBJ, OBJ);

      inline CondEquals( const A _a, const B _b):ICondition(), M_a(_a), M_b(_b){ }
      inline CondEquals( const CondEquals& other ):ICondition(), M_a(other.M_a), M_b(other.M_b) { }
      
      inline ~CondEquals(){}
      
      inline CondEquals<A,B>* clone() const = 0;
      
      inline CondEquals<A,B>& operator=(const CondEquals<A,B>& other) {
          if (this != &other) {
              M_a = other.M_a;
              M_b = other.M_b;
              // f = other.f;
          }
          return *this;
      }
      
      inline bool operator==(const CondEquals<A,B>& other) {
          if (this != &other) {
              return  M_a == other.M_a && M_b == other.M_b; // && f == other.f
          }
          return true;
      }
      
      // Accessors
      const A a() { return M_a; }
      const B b() { return M_b; }
      // const EqualityFunc f() { return f; }

      private:
          const A M_a;
          const B M_b;
          // EqualityFunc f;
    };

}

#endif
