#include "cond_object_pos.h"

#include <setplay/runtime/icondition_evaluator.h>
#include <setplay/model/geom/ilocation.h>
#include <setplay/model/ivisitor.h>
#include <setplay/model/object.h>

namespace setplay {

    CondAtLocation::CondAtLocation( IMobileObject *object,
                                    ILocation2D *location)
        : M_object( object)
        , M_location( location )
    {

    }

    CondAtLocation::CondAtLocation( const CondAtLocation& other )
    {
        M_object = other.M_object != NULL ? other.M_object->clone() : NULL;
        M_location = other.M_location != NULL ? other.M_location->clone() : NULL;
    }

    CondAtLocation::~CondAtLocation()
    {
        if ( M_object != NULL )
        {
            delete M_object;
        }
        if ( M_location != NULL )
        {
            delete M_location;
        }
    }

    TokenType CondAtLocation::type() const
    {
        return TT_CONDITION_OBJECT_LOCATION;
    }

    IMobileObject* CondAtLocation::object() const
    {
        return M_object;
    }

    ILocation2D* CondAtLocation::location() const
    {
        return M_location;
    }

    CondAtLocation& CondAtLocation::setObject(IMobileObject* object)
    {
        if ( M_object != NULL ) { delete M_object; }
        M_object = object;
        return *this;
    }

    CondAtLocation& CondAtLocation::setLocation(ILocation2D* location)
    {
        if ( M_location != NULL ) { delete M_location; }
        M_location = location;
        return *this;
    }

    CondAtLocation& CondAtLocation::operator=(const CondAtLocation& other)
    {
        if ( this != &other)
        {
            // TODO Not checking for nulls
            setObject( other.M_object->clone() );
            setLocation( other.M_location->clone() );
        }
        return *this;
    }

    CondAtLocation* CondAtLocation::clone() const
    {
        return new CondAtLocation( *this );
    }

    CondAtLocation& CondAtLocation::operator*()
    {
        return *this;
    }

    void CondAtLocation::visit(IVisitor& visitor)
    {
        visitor.visit( *this );
    }

}
