#include "functor_point.h"

namespace setplay {

/// VectorNormalize class implementation

VectorNormalize::VectorNormalize( IPoint2D* operand)
    : UnaryOperation(operand)
{

}

VectorNormalize::VectorNormalize( const VectorNormalize& other)
    : UnaryOperation(other)
{

}

TokenType VectorNormalize::type() const
{
    return TT_VECTOR_NORMALIZE;
}

void VectorNormalize::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

VectorNormalize& VectorNormalize::operator*()
{
    return *this;
}

VectorNormalize* VectorNormalize::clone() const
{
    return new VectorNormalize( *this );
}

/// VectorInvertX class implementation

VectorInvertX::VectorInvertX( IPoint2D* operand)
    : UnaryOperation(operand)
{

}

VectorInvertX::VectorInvertX( const VectorInvertX& other)
    : UnaryOperation(other)
{

}

TokenType VectorInvertX::type() const
{
    return TT_VECTOR_INVERT_X;
}

void VectorInvertX::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

VectorInvertX& VectorInvertX::operator*()
{
    return *this;
}

VectorInvertX* VectorInvertX::clone() const
{
    return new VectorInvertX( *this );
}

/// VectorInvertY class implementation

VectorInvertY::VectorInvertY( IPoint2D* operand)
    : UnaryOperation(operand)
{

}

VectorInvertY::VectorInvertY( const VectorInvertY& other)
    : UnaryOperation(other)
{

}

TokenType VectorInvertY::type() const
{
    return TT_VECTOR_INVERT_Y;
}

void VectorInvertY::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

VectorInvertY& VectorInvertY::operator*()
{
    return *this;
}

VectorInvertY* VectorInvertY::clone() const
{
    return new VectorInvertY( *this );
}

/// VectorInvertXY class implementation

VectorInvertXY::VectorInvertXY( IPoint2D* operand)
    : UnaryOperation(operand)
{

}

VectorInvertXY::VectorInvertXY( const VectorInvertXY& other)
    : UnaryOperation(other)
{

}

TokenType VectorInvertXY::type() const
{
    return TT_VECTOR_INVERT_XY;
}

void VectorInvertXY::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

VectorInvertXY& VectorInvertXY::operator*()
{
    return *this;
}

VectorInvertXY* VectorInvertXY::clone() const
{
    return new VectorInvertXY( *this );
}

/// VectorSubtraction Operation implementation

VectorSubtraction::VectorSubtraction( IPoint2D* operand_left,
                                      IPoint2D* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorSubtraction::VectorSubtraction( const VectorSubtraction& other)
    : BinaryOperation(other)
{

}

TokenType VectorSubtraction::type() const
{
    return TT_VECTOR_SUBTRACTION;
}


void VectorSubtraction::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}


VectorSubtraction& VectorSubtraction::operator*()
{
    return *this;
}

VectorSubtraction* VectorSubtraction::clone() const
{
    return new VectorSubtraction( *this );
}

/// VectorAddition Operation implementation

VectorAddition::VectorAddition( IPoint2D* operand_left,
                                IPoint2D* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorAddition::VectorAddition( const VectorAddition& other)
    : BinaryOperation(other)
{

}

TokenType VectorAddition::type() const
{
    return TT_VECTOR_ADDITION;
}


void VectorAddition::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

VectorAddition& VectorAddition::operator*()
{
    return *this;
}

VectorAddition* VectorAddition::clone() const
{
    return new VectorAddition( *this );
}

/// VectorScale Operation implementation

VectorScale::VectorScale( IPoint2D* operand_left,
                          IDecimal* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorScale::VectorScale( const VectorScale& other)
    : BinaryOperation(other)
{

}

TokenType VectorScale::type() const
{
    return TT_VECTOR_SCALE;
}


void VectorScale::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

VectorScale& VectorScale::operator*()
{
    return *this;
}

VectorScale* VectorScale::clone() const
{
    return new VectorScale( *this );
}

/// VectorDivision Operation implementation

VectorDivision::VectorDivision( IPoint2D* operand_left,
                                IDecimal* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorDivision::VectorDivision( const VectorDivision& other)
    : BinaryOperation(other)
{

}

TokenType VectorDivision::type() const
{
    return TT_VECTOR_DIVISION;
}


void VectorDivision::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

VectorDivision& VectorDivision::operator*()
{
    return *this;
}

VectorDivision* VectorDivision::clone() const
{
    return new VectorDivision( *this );
}

/// VectorRotation Operation implementation

VectorRotation::VectorRotation( IPoint2D* operand_left,
                                IDecimal* operand_right)
    : BinaryOperation(operand_left, operand_right)
{

}

VectorRotation::VectorRotation( const VectorRotation& other)
    : BinaryOperation(other)
{

}

TokenType VectorRotation::type() const
{
    return TT_VECTOR_ROTATION;
}


void VectorRotation::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

VectorRotation& VectorRotation::operator*()
{
    return *this;
}

VectorRotation* VectorRotation::clone() const
{
    return new VectorRotation( *this );
}
/// NearestIntersectionPoint binary operation implementation

NearestIntersectionPoint::NearestIntersectionPoint( IPoint2D* operand_left, IRegion2D* operand_right )
    : BinaryOperation(operand_left, operand_right)
{

}

NearestIntersectionPoint::NearestIntersectionPoint( const NearestIntersectionPoint& other )
    : BinaryOperation(other)
{

}

TokenType NearestIntersectionPoint::type() const
{
    return TT_NEAREST_INTERSECTION_POINT;
}

void NearestIntersectionPoint::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

NearestIntersectionPoint& NearestIntersectionPoint::operator*()
{
    return *this;
}

NearestIntersectionPoint* NearestIntersectionPoint::clone() const
{
    return new NearestIntersectionPoint( *this );
}

/// RegionCentroid binary operation implementation

RegionCentroid::RegionCentroid( IRegion2D* operand )
    : UnaryOperation(operand)
{

}

RegionCentroid::RegionCentroid( const RegionCentroid& other )
    : UnaryOperation(other)
{

}

TokenType RegionCentroid::type() const
{
    return TT_REGION_CENTROID;
}

void RegionCentroid::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

RegionCentroid& RegionCentroid::operator*()
{
    return *this;
}

RegionCentroid* RegionCentroid::clone() const
{
    return new RegionCentroid( *this );
}

/// MobileObjectLocation
MobileObjectLocation::MobileObjectLocation( IMobileObject* mobile_object )
    : M_mobile_object( mobile_object )
{

}

MobileObjectLocation::MobileObjectLocation( const MobileObjectLocation& other )
{
    M_mobile_object = other.M_mobile_object != NULL
                        ? other.M_mobile_object->clone()
                        : NULL;
}

MobileObjectLocation::~MobileObjectLocation()
{
    if ( M_mobile_object != NULL )
    {
        delete M_mobile_object;
    }
}

TokenType MobileObjectLocation::type() const
{
    return TT_MOBILE_OBJECT_LOCATION;
}

IMobileObject* MobileObjectLocation::mobileObject() const
{
    return M_mobile_object;
}

MobileObjectLocation& MobileObjectLocation::operator*()
{
    return *this;
}

MobileObjectLocation* MobileObjectLocation::clone() const
{
    return new MobileObjectLocation( *this );
}

void MobileObjectLocation::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}


/// PlayerPosition implementation

PlayerPosition::PlayerPosition( IPlayer* player )
    : M_player(player)
{

}

PlayerPosition::PlayerPosition( const PlayerPosition& other)
{
    M_player = other.M_player != NULL ? other.M_player->clone() : NULL;
}

PlayerPosition::~PlayerPosition()
{
    if ( M_player != NULL ) { delete M_player; }
}

TokenType PlayerPosition::type() const
{
    return TT_PLAYER_POSITION;
}

IPlayer* PlayerPosition::player() const
{
    return M_player;
}

PlayerPosition& PlayerPosition::setPlayer(IPlayer* player)
{
    if ( M_player != NULL ) { delete M_player; }
    M_player = player;
    return *this;
}

PlayerPosition& PlayerPosition::operator=(const PlayerPosition& other)
{
    if ( this != &other )
    {
        setPlayer(other.M_player != NULL ? other.M_player->clone() : NULL);
    }
    return *this;
}

PlayerPosition& PlayerPosition::operator*()
{
    return *this;
}

PlayerPosition* PlayerPosition::clone() const
{
    return new PlayerPosition(*this);
}

void PlayerPosition::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

/// BallPosition implementation

BallPosition::BallPosition()
{

}

BallPosition::~BallPosition()
{

}

BallPosition& BallPosition::operator=(const BallPosition& )
{
    return *this;
}

TokenType BallPosition::type() const
{
    return TT_BALL_POSITION;
}

BallPosition& BallPosition::operator*()
{
    return *this;
}

BallPosition* BallPosition::clone() const
{
    return new BallPosition();
}

void BallPosition::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

}
