#ifndef FUNCTOR_POINT_H
#define FUNCTOR_POINT_H

#include <setplay/model/operation.h>
#include <setplay/model/participants/iplayer.h>
#include <setplay/model/types.h>
#include <setplay/model/geom/ipoint.h>
#include <setplay/model/geom/iregion.h>

namespace setplay
{

/*!
  \brief VectorNormalize class
  */
class VectorNormalize
        : public IPoint2D
        , public UnaryOperation<VectorNormalize, IPoint2D>
{
public:
    VectorNormalize( IPoint2D *operand = NULL );
    VectorNormalize( const VectorNormalize& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorNormalize& operator*();
    VectorNormalize* clone() const;
};

/*!
  \brief VectorInvertX class
  */
class VectorInvertX
        : public IPoint2D
        , public UnaryOperation<VectorInvertX, IPoint2D>
{
public:
    VectorInvertX( IPoint2D *operand = NULL );
    VectorInvertX( const VectorInvertX& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorInvertX& operator*();
    VectorInvertX* clone() const;
};

/*!
  \brief VectorInvertX class
  */
class VectorInvertY
        : public IPoint2D
        , public UnaryOperation<VectorInvertY, IPoint2D>
{
public:
    VectorInvertY( IPoint2D *operand = NULL);
    VectorInvertY( const VectorInvertY& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorInvertY& operator*();
    VectorInvertY* clone() const;
};


/*!
  \brief VectorInvertXY class
  */
class VectorInvertXY
        : public IPoint2D
        , public UnaryOperation<VectorInvertXY, IPoint2D>
{
public:
    VectorInvertXY( IPoint2D *operand = NULL);
    VectorInvertXY( const VectorInvertXY& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorInvertXY& operator*();
    VectorInvertXY* clone() const;
};


/*!
  \brief VectorSubtraction class
  */
class VectorSubtraction
        : public IPoint2D
        , public BinaryOperation<VectorSubtraction, IPoint2D>
{
public:
    VectorSubtraction( IPoint2D* operand_left = NULL, IPoint2D* operand_right = NULL);
    VectorSubtraction( const VectorSubtraction& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorSubtraction& operator*();
    VectorSubtraction* clone() const;
};

/*!
  \brief VectorAddition class
  */
class VectorAddition
        : public IPoint2D
        , public BinaryOperation<VectorAddition, IPoint2D>
{
public:
    VectorAddition( IPoint2D* operand_left = NULL, IPoint2D* operand_right = NULL);
    VectorAddition( const VectorAddition& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorAddition& operator*();
    VectorAddition* clone() const;
};

/*!
  \brief VectorScale class
  */
class VectorScale
        : public IPoint2D
        , public BinaryOperation<VectorScale, IPoint2D, IDecimal>
{
public:
    VectorScale( IPoint2D* operand_left = NULL, IDecimal* operand_right = NULL);
    VectorScale( const VectorScale& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorScale& operator*();
    VectorScale* clone() const;
};

/*!
  \brief VectorRotation class
  */
class VectorRotation
        : public IPoint2D
        , public BinaryOperation<VectorRotation, IPoint2D, IDecimal>
{
public:
    VectorRotation( IPoint2D* operand_left, IDecimal* operand_right);
    VectorRotation( const VectorRotation& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorRotation& operator*();
    VectorRotation* clone() const;
};

/*!
  \brief VectorDivision class
  */
class VectorDivision
        : public IPoint2D
        , public BinaryOperation<VectorDivision, IPoint2D, IDecimal>
{
public:
    VectorDivision( IPoint2D* operand_left = NULL, IDecimal* operand_right = NULL);
    VectorDivision( const VectorDivision& other);
    TokenType type() const;

    void visit(IVisitor &visitor);
    VectorDivision& operator*();
    VectorDivision* clone() const;
};



/*!
  \brief Determine the nearest intersection point between a point and a region
  */
class NearestIntersectionPoint
        : public IPoint2D
        , public BinaryOperation<NearestIntersectionPoint, IPoint2D, IRegion2D>
{
public:
    NearestIntersectionPoint( IPoint2D* operand_left = NULL, IRegion2D* operand_right = NULL );
    NearestIntersectionPoint( const NearestIntersectionPoint& other );

    TokenType type() const;

    void visit(IVisitor &visitor);
    NearestIntersectionPoint& operator*();
    NearestIntersectionPoint* clone() const;
};


/*!
  \brief Determine the centroid of a region
  */
class RegionCentroid
        : public IPoint2D
        , public UnaryOperation<RegionCentroid, IRegion2D>
{
public:
    RegionCentroid( IRegion2D* operand = NULL );
    RegionCentroid( const RegionCentroid& other );

    TokenType type() const;

    void visit(IVisitor &visitor);
    RegionCentroid& operator*();
    RegionCentroid* clone() const;
};

class MobileObjectLocation
        : public IPoint2D
{
public:
    MobileObjectLocation( IMobileObject* mobile_object = NULL);
    MobileObjectLocation( const MobileObjectLocation& other );
    ~MobileObjectLocation();

    TokenType type() const;
    IMobileObject *mobileObject() const;

    MobileObjectLocation& operator*();
    MobileObjectLocation* clone() const;
    void visit(IVisitor &visitor);
private:
    IMobileObject* M_mobile_object;
};

/*!
  \brief Class to represent the position of a given player
  */
class PlayerPosition
        : public IPoint2D
{
public:
    PlayerPosition( IPlayer* player = NULL);
    PlayerPosition( const PlayerPosition& other);
    ~PlayerPosition();

    IPlayer *player() const;

    TokenType type() const;

    PlayerPosition& setPlayer(IPlayer* player);
    PlayerPosition& operator=(const PlayerPosition& other);

    PlayerPosition* clone() const;
    PlayerPosition& operator*();
    void visit(IVisitor &visitor);
private:
    IPlayer* M_player;
};

/*!
  \brief Class to represent the position of the ball
  */
class BallPosition
        : public IPoint2D
{
public:
    BallPosition();
    ~BallPosition();

    BallPosition& operator=(const BallPosition& );

    TokenType type() const;

    BallPosition* clone() const;
    BallPosition& operator*();
    void visit(IVisitor &visitor);
};


}


#endif // FUNCTOR_POINT_H
