#ifndef IVISITOR_H
#define IVISITOR_H


namespace setplay {

    class Setplay;
    class Step;

    class IParticipation;

    /// Directives
    class Do;
    class DoWhile;
    class Dont;

    /// Transitions
    class NextStep;
    class Finish;
    class Abort;

    /// Actions
    class ActDribble;
    class ActHold;
    class ActFollow;
    class ActIntercept;
    class ActPass;
    class ActPos;
    class ActShoot;
    class ActLookAt;

    /// Conditions
    class CondAnd;
    class CondOr;
    class CondNot;
    class CondBallOwner;
    class CondTeamInPossession;
    class CondBool;
    class CondGameSituation;
    class CondTeamFormation;
    class CondAtLocation;
    class CondOffside;
    class CondCompDecimal;
    class CondKickable;
    class CondCanIntercept;
    class CondCanShoot;
    class CondCanPass;

    /// Regions
    class Rect2D;
    class Triangle2D;
    class Sector2D;
    class Circle2D;
    class Ray2D;
    class Line2D;

    /// Participants
    class PlayerAffiliation;

    /// Variables
    class Variable;
    class VarLocation;
    class VarPoint;
    class VarRegion;
    class VarPlayer;
    class VarCondition;
    class VarDecimal;
    class VarInteger;

    /// Points
    class Point2D;

    /// Point Functors
    // Vector unary operations
    class VectorNormalize;
    class VectorInvertX;
    class VectorInvertY;
    class VectorInvertXY;
    class VectorSubtraction;
    class VectorAddition;
    // Vector scalar operations
    class VectorScale;
    class VectorRotation;
    class VectorDivision;
    class NearestIntersectionPoint;
    class RegionCentroid;
    class MobileObjectLocation;
    class PlayerPosition;
    class BallPosition;

    /// Number Functors
    // Number operations
    class NumberAbs;
    class NumberSignChange;
    class NumberMultiplication;
    class NumberSubtraction;
    class NumberDivision;
    class NumberAddition;
    // Vector to scalar operations
    class VectorDirection;
    class VectorLength;
    // Two vector to scalar operations
    class VectorDistance;
    class VectorAngularDiff;
    class VectorInnerProduct;
    class VectorOuterProduct;
    // In game queries
    class DefenseLine;
    class OffenseLine;
    class OffsideLine;
    class MobileObjectSpeed;

    class PointX;
    class PointY;

    class CurrentTime;
    class RemainingTime;
    class GoalsScored;
    class GoalDiff;
    class InterceptionTime;

    /// Player Functors
    class PlayerSelf;
    class NearestPlayer;
    class NearestTeammate;
    class NearestOpponent;
    class Goalie;

    /// Numbers
    class Decimal;
    class Integer;
    class Bool;

    class Assignment;

    /// MobileObject
    class Ball;

    class IVisitor {
    public:
        virtual ~IVisitor() { }


        virtual void visit( Setplay& setplay) = 0;
        virtual void visit( Step& step) = 0;
        virtual void visit( IParticipation& participation) = 0;

        /// Transitions
        virtual void visit( NextStep& transition) = 0;
        virtual void visit( Finish& transition) = 0;
        virtual void visit( Abort& transition) = 0;

        /// Directives
        virtual void visit( Do& directive) = 0;
        virtual void visit( DoWhile& directive) = 0;
        virtual void visit( Dont& directive) = 0;

        /// Actions
        virtual void visit( ActDribble& action) = 0;
        virtual void visit( ActFollow& action) = 0;
        virtual void visit( ActHold& action) = 0;
        virtual void visit( ActIntercept& action) = 0;
        virtual void visit( ActPass& action) = 0;
        virtual void visit( ActPos& action) = 0;
        virtual void visit( ActShoot& action) = 0;
        virtual void visit( ActLookAt& action) = 0;

        /// Conditions
        virtual void visit( CondAnd& condition) = 0;
        virtual void visit( CondOr& condition) = 0;
        virtual void visit( CondNot& condition) = 0;
        virtual void visit( CondBallOwner& condition) = 0;
        virtual void visit( CondTeamInPossession& condition) = 0;
        virtual void visit( CondBool& condition) = 0;
        virtual void visit( CondGameSituation& condition) = 0;
        virtual void visit( CondAtLocation& condition) = 0;
        virtual void visit( CondTeamFormation& condition) = 0;
        virtual void visit( CondOffside& condition) = 0;
        virtual void visit( CondCompDecimal& condition) = 0;
        virtual void visit( CondKickable& condition) = 0;
        virtual void visit( CondCanIntercept& condition) = 0;
        virtual void visit( CondCanShoot& condition) = 0;
        virtual void visit( CondCanPass& condition) = 0;

        /// Locations
        virtual void visit( Rect2D& location) = 0;
        virtual void visit( Triangle2D& location) = 0;
        virtual void visit( Sector2D& location) = 0;
        virtual void visit( Circle2D& location) = 0;
        virtual void visit( Ray2D& location) = 0;
        virtual void visit( Line2D& location) = 0;

        /// Participant
        virtual void visit( PlayerAffiliation& participant) = 0;

        /// Player functors
        virtual void visit( PlayerSelf& player_functor) = 0;
        virtual void visit( NearestPlayer& player_functor) = 0;
        virtual void visit( NearestTeammate& player_functor) = 0;
        virtual void visit( NearestOpponent& player_functor) = 0;
        virtual void visit( Goalie& player_functor) = 0;

        /// Points
        virtual void visit( Point2D& point) = 0;

        /// Variables => Is this really necessary??
        virtual void visit( Variable& variable) = 0;
        virtual void visit( VarLocation& variable) = 0;
        virtual void visit( VarRegion& variable) = 0;
        virtual void visit( VarPoint& variable) = 0;
        virtual void visit( VarCondition& variable) = 0;
        virtual void visit( VarDecimal& variable) = 0;
        virtual void visit( VarInteger& variable) = 0;
        virtual void visit( VarPlayer& variable) = 0;

        /// Point Functors
        virtual void visit( MobileObjectLocation& point_functor) = 0;
        virtual void visit( PlayerPosition& point_functor) = 0;
        virtual void visit( BallPosition& point_functor) = 0;
        // Vector unary operations
        virtual void visit( VectorNormalize& point_functor) = 0;
        virtual void visit( VectorInvertX& point_functor) = 0;
        virtual void visit( VectorInvertY& point_functor) = 0;
        virtual void visit( VectorInvertXY& point_functor) = 0;
        virtual void visit( VectorSubtraction& point_functor) = 0;
        virtual void visit( VectorAddition& point_functor) = 0;
        // Vector scalar operations
        virtual void visit( VectorScale& point_functor) = 0;
        virtual void visit( VectorRotation& point_functor) = 0;
        virtual void visit( VectorDivision& point_functor) = 0;
        virtual void visit( NearestIntersectionPoint& point_functor) = 0;
        virtual void visit( RegionCentroid& point_functor) = 0;

        /// Simple Data Types
        virtual void visit( Decimal& number) = 0;
        virtual void visit( Integer& number) = 0;
        virtual void visit( Bool& number ) = 0;

        /// Number Functors
        // Number operations
        virtual void visit( NumberAbs& decimal_functor) = 0;
        virtual void visit( NumberSignChange& decimal_functor) = 0;
        virtual void visit( NumberMultiplication& decimal_functor) = 0;
        virtual void visit( NumberSubtraction& decimal_functor) = 0;
        virtual void visit( NumberDivision& decimal_functor) = 0;
        virtual void visit( NumberAddition& decimal_functor) = 0;
        // Vector to scalar operations
        virtual void visit( VectorDirection& decimal_functor) = 0;
        virtual void visit( VectorLength& decimal_functor) = 0;
        // Two vector to scalar operations
        virtual void visit( VectorDistance& decimal_functor) = 0;
        virtual void visit( VectorAngularDiff& decimal_functor) = 0;
        virtual void visit( VectorInnerProduct& decimal_functor) = 0;
        virtual void visit( VectorOuterProduct& decimal_functor) = 0;
        // In game queries
        virtual void visit( DefenseLine& decimal_functor) = 0;
        virtual void visit( OffenseLine& decimal_functor) = 0;
        virtual void visit( OffsideLine& decimal_functor) = 0;
        virtual void visit( MobileObjectSpeed& decimal_functor) = 0;

        virtual void visit( PointX& decimal_functor ) = 0;
        virtual void visit( PointY& decimal_functor ) = 0;

        /// Integer Functors
        virtual void visit( CurrentTime& integer_functor) = 0;
        virtual void visit( RemainingTime& integer_functor) = 0;
        virtual void visit( GoalsScored& integer_functor) = 0;
        virtual void visit( GoalDiff& integer_functor) = 0;
        virtual void visit( InterceptionTime& integer_functor ) = 0;

        /// Assignment function
        virtual void visit( Assignment& assign) = 0;

        virtual void visit( Ball& assign) = 0;

    };

}

#endif // IVISITOR_H
