#include "rewrite_rule.h"

#include <setplay/model/geom/point_2d.h>
#include <setplay/model/conditions/cond_comp_decimal.h>
#include <setplay/model/geom/functor_number.h>
#include <setplay/runtime/setplay_executor.h>

namespace setplay {

RewritePlayerPosition::RewritePlayerPosition(SetplayInterpreter *interpreter)
    : M_interpreter(interpreter)
{
    assert ( interpreter != NULL );
}

bool RewritePlayerPosition::accepts(const Object &setplay_concept)
{
    return setplay_concept.type() == TT_PLAYER_POSITION;
}

Object* RewritePlayerPosition::rewrite(const Object& setplay_concept)
{
    const PlayerPosition& player_position = static_cast<const PlayerPosition&>(setplay_concept);

    rcsc::Vector2D player_pos = M_interpreter->interpret( player_position );
    return new Point2D( new Decimal(player_pos.x),
                        new Decimal(player_pos.y) );
}

/// RewriteBallPosition

RewriteBallPosition::RewriteBallPosition(SetplayInterpreter *interpreter)
    : M_interpreter(interpreter)
{
    assert ( interpreter != NULL );
}

bool RewriteBallPosition::accepts(const Object &setplay_concept)
{
    return setplay_concept.type() == TT_BALL_POSITION;
}

Object* RewriteBallPosition::rewrite(const Object& setplay_concept)
{
    const BallPosition& ball_position = static_cast<const BallPosition&>(setplay_concept);

    rcsc::Vector2D ball_pos= M_interpreter->interpret( ball_position );
    return new Point2D( new Decimal(ball_pos.x),
                        new Decimal(ball_pos.y) );
}


/// RewriteDecimalBinaryOperation

RewriteDecimalBinaryOperation::RewriteDecimalBinaryOperation(SetplayInterpreter *interpreter)
    : M_interpreter(interpreter)
{
    assert ( interpreter != NULL );
}

bool RewriteDecimalBinaryOperation::accepts(const Object &setplay_concept)
{
    return setplay_concept.type() == TT_TWO_DECIMAL_TO_DECIMAL_OPERATION;
}

Object* RewriteDecimalBinaryOperation::rewrite(const Object& setplay_concept)
{
    const TwoDecimalToDecimalOperation& decimal_bin_opp = static_cast<const TwoDecimalToDecimalOperation&>(setplay_concept);

    return new Decimal( M_interpreter->interpret( decimal_bin_opp ) );
}

/// RewriteDynamicPlayer

RewriteDynamicPlayer::RewriteDynamicPlayer(SetplayInterpreter *interpreter)
    : M_interpreter(interpreter)
{
    assert ( interpreter != NULL );
}

bool RewriteDynamicPlayer::accepts(const Object &setplay_concept)
{
    return setplay_concept.type() == TT_VAR_PLAYER ||
           setplay_concept.type() == TT_NEAREST_PLAYER;

}

Object* RewriteDynamicPlayer::rewrite(const Object& setplay_concept)
{
    if ( setplay_concept.type() == TT_VAR_PLAYER )
    {
        const VarPlayer& var_player = static_cast<const VarPlayer&>(setplay_concept);
        return new PlayerAffiliation( M_interpreter->interpret( var_player ) );
    }

    const NearestPlayer& nearest_player = static_cast<const NearestPlayer&>(setplay_concept);
    return new PlayerAffiliation( M_interpreter->interpret( nearest_player ) );
}


/// RewriteCondWaitTime

RewriteCondWaitTime::RewriteCondWaitTime(SetplayInterpreter *interpreter)
    : M_interpreter(interpreter)
{
    assert ( interpreter != NULL );
}

bool RewriteCondWaitTime::accepts(const Object &setplay_concept)
{
    return setplay_concept.type() == TT_COND_WAIT_TIME;
}

Object* RewriteCondWaitTime::rewrite(const Object& setplay_concept)
{
    const CondWaitTime& condition = static_cast<const CondWaitTime&>(setplay_concept);

    double wait_time = M_interpreter->interpret( *condition.waitTime() );
    double current_time = M_interpreter->interpret( RemainingTime() );

    // wait(5) => time() + 5 > time() OR time() < time() + 5
    return new CondCompDecimal( DCOT_GREATER,
                                new Decimal( current_time + wait_time ),
                                new RemainingTime());
}

}
