#ifndef SEPTLAY_SCOPE_H
#define SEPTLAY_SCOPE_H

#include <string>
#include <map>

#include <setplay/model/variable.h>

namespace setplay
{
    class Object;

    class IScope
    {
    public:
        virtual ~IScope() { }

        virtual const std::string& name() const = 0;
        virtual IScope* parent() = 0;
        virtual bool define(Symbol* sym) = 0;
        virtual Symbol* resolve(std::string name) = 0;
    };

    struct ScopeNameComparator: std::less<IScope*>
    {
      bool operator()(const IScope*& lhs, const IScope*& rhs) const
      {
          return lhs->name().compare( rhs->name()) < 0;
      }
    };


    class GlobalScope
            : public IScope
    {
    public:
        GlobalScope();
        const std::string& name() const;
        IScope* parent();
        bool define(Symbol* sym);
        Symbol* resolve(std::string name);
    private:
        std::map<std::string, Symbol*> M_symbols;
    };

    /*!
      \brief Use this class to create a scope foreach set-play objects. Each refers to an existing parent space
      */
    class Scope
            : public IScope
    {
    public:
        /*!
          \param owner_object The AST object to which this scope belongs to
          \param name The name to be used to identify the scope (for debugging purposes)
          \param parent The parent scope of this scope
          */
        Scope(Object* owner_object, std::string name, IScope* parent = NULL);
        const std::string& name() const;
        IScope* parent();
        bool define(Symbol* sym);
        Symbol* resolve(std::string name);

    private:
        Object* M_owner_object;
        std::string M_name;
        IScope* M_parent;
        std::map<std::string, Symbol*> M_symbols;
    };

}

#endif // SEPTLAY_SCOPE_H
