#ifndef SETPLAY_TYPES_H
#define SETPLAY_TYPES_H

#include <setplay/model/object.h>

namespace setplay {


    typedef unsigned int Unum;

    enum TeamAffiliation
    {
        TA_UNKNOWN,
        TA_OUR,
        TA_OPP
    };

    enum TimePeriod
    {
        TP_FIRST_HALF,
        TP_SECOND_HALF,
        TP_EXTRA_TIME_FIRST_HALF,
        TP_EXTRA_TIME_SECOND_HALF
    };

    enum SoccerMomentAffiliation
    {
        SM_GAME_STOPPED_THEIR_POSSESION,
        SM_GAME_STOPPED_OUR_POSSESION,
        SM_OUR_POSSESION,
        SM_THEIR_POSSESION,
        SM_LOSING_POSSESION,
        SM_REGAINING_POSSESION
    };

    class Bool
                : public Object
	{
            public:
			Bool( bool value );
			Bool(const Bool&other);
			~Bool();
			

            TokenType type() const;

			bool value() const;
            Bool& setValue(bool value);


            // Comparators
            bool operator==(const Bool& other);
            bool operator!=(const Bool& other);

			Bool& operator=(const Bool& other);

            Bool* clone() const;
			void visit(IVisitor& visitor);

            private:
                bool M_value;
	};

    template <typename T>
    class Number
    {
    public:
        Number(T value)
            : M_value( value)
        {

        }

        Number( const Number& other )
            : M_value( other.M_value )
        {

        }

        ~Number()
        {

        }

        T value() const
        {
            return M_value;
        }

        Number& setValue(T value)
        {
            M_value = value;
        }

        Number& operator=(const Number& other)
        {
            M_value = other.M_value;
            return *this;
        }

        bool operator==(const Number& other) const
        {
            return M_value == other.M_value;
        }

        bool operator!=(const Number& other) const
        {
            return !(*this == other);
        }

        bool operator<(const Number& other) const
        {
            return M_value < other.M_value;
        }

        bool operator>=(const Number& other) const
        {
            return !(*this < other);
        }

        bool operator>(const Number& other) const
        {
            return M_value > other.M_value;
        }

        bool operator<=(const Number& other) const
        {
            return !(*this > other);
        }

    private:
        T M_value;
    };


    class INumber
            : public Object
    {
    public:
        typedef enum
        {
            NT_INTEGER,
            NT_INTEGER_VAR,
            NT_DECIMAL,
            NT_DECIMAL_VAR
        } NumberType;

        virtual ~INumber() { }

        virtual INumber* clone() const = 0;
        virtual INumber& operator*() = 0;
    };

    // INTEGER VALUES

    class IInteger
            : public INumber
    {
    public:
        virtual ~IInteger() { }

        virtual IInteger* clone() const = 0;
        virtual IInteger& operator*() = 0;
    };

    class Integer
            : public Number<int>
            , public IInteger
    {
    public:
        Integer(int value);
        Integer(const Integer& other);
        ~Integer();

        TokenType type() const;

        Integer* clone() const;
        Integer& operator*();
        void visit(IVisitor& visitor);
    };


    class IDecimal
            : public INumber
    {
    public:
        virtual ~IDecimal() { }

        virtual IDecimal* clone() const = 0;
        virtual IDecimal& operator*() = 0;
    };

    class Decimal
            : public Number<float>
            , public IDecimal
    {
    public:
        Decimal(float value);
        Decimal(const Decimal& other);
        ~Decimal();

        TokenType type() const;

        void visit(IVisitor& visitor);
        Decimal* clone() const;
        Decimal& operator*();
    };

}

#endif
