#include "SetplayParserSExpr.h"

#include <iostream>
#include <string>

#include "../conditions/cond_bool.h"

// OLD PARSING - LMOTA
#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_assign_actor.hpp>

// OLD
using namespace boost;
using namespace BOOST_SPIRIT_CLASSIC_NS;

using namespace std;

namespace setplay {

    const string& SetplayParserSExpr::setplay_element = "setplay";
    const string& SetplayParserSExpr::step_element = "step";
    const string& SetplayParserSExpr::next_step_element = "nextstep";
    const string& SetplayParserSExpr::abort_transition_element = "abort";
    const string& SetplayParserSExpr::finish_transition_element = "finish";
    const string& SetplayParserSExpr::abort_condition_attribute = "abortcond";
    const string& SetplayParserSExpr::start_condition_attribute = "condition";
    const string& SetplayParserSExpr::abort_time_attribute = "abortTime";
    const string& SetplayParserSExpr::wait_time_attribute = "waitTime";
    const string& SetplayParserSExpr::directives_attribute = "directives";
    const string& SetplayParserSExpr::transitions_attribute = "transitions";
    const string& SetplayParserSExpr::steps_attribute = "steps";
    const string& SetplayParserSExpr::name_attribute = "name";
    const string& SetplayParserSExpr::comment_attribute = "comment";
    const string& SetplayParserSExpr::id_attribute = "id";
    const string& SetplayParserSExpr::invertable_attribute = "invertable";
    const string& SetplayParserSExpr::participants_attribute = "participants";
    const string& SetplayParserSExpr::participations_attribute = "participants";
    const string& SetplayParserSExpr::leader_attribute = "leader";
    const string& SetplayParserSExpr::next_step_attribute = "nextStep";

    // Participations
    const string& SetplayParserSExpr::player_element = "player";
    const string& SetplayParserSExpr::player_var = "playerRole";
    const string& SetplayParserSExpr::player_number_attribute = "number";
    const string& SetplayParserSExpr::player_team_attribute = "team";
    const string& SetplayParserSExpr::participation_element = "at";

    // Action elements
    const string& SetplayParserSExpr::act_pos_element = "pos";
    const string& SetplayParserSExpr::act_pass_element = "pass";
    const string& SetplayParserSExpr::act_dribble_element = "dribble";
    const string& SetplayParserSExpr::act_intercept_element = "intercept";
    const string& SetplayParserSExpr::act_hold_element = "hold";
    const string& SetplayParserSExpr::act_shoot_element = "shoot";

    // Directives
    const string& SetplayParserSExpr::directive_do = "do";
    // const string& SetplayParserSExpr::directive_do = "dont"; // NOT USED EVER => DISREGARD
    const string& SetplayParserSExpr::player_attribute = "player";
    const string& SetplayParserSExpr::actions_attribute = "actions";

    // Regions
    const string& SetplayParserSExpr::region_variable_element = "regvar";
    const string& SetplayParserSExpr::region_arc_element = "arc"; // NOT USED EVER
    const string& SetplayParserSExpr::region_triangle_element = "triangle"; // NOT USED EVER
    const string& SetplayParserSExpr::region_rect_element = "rec";

    // Points
    const string& SetplayParserSExpr::point_var_element = "ptvar";
    const string& SetplayParserSExpr::point_element = "pt";
    const string& SetplayParserSExpr::point_ball_element = "ptball";
    const string& SetplayParserSExpr::point_player_element = "ptplayer";
    const string& SetplayParserSExpr::point_rel_element = "ptrel";
    const string& SetplayParserSExpr::point_x_attribute = "x";
    const string& SetplayParserSExpr::point_y_attribute = "y";
    // const string& SetplayParserSExpr::point_arith_element = ""; // NOT USED EVER

    // Conditions
    const string& SetplayParserSExpr::cond_player_pos_element = "ppos";
    const string& SetplayParserSExpr::cond_ball_pos_element = "bpos";
    const string& SetplayParserSExpr::cond_ball_owner_element = "bowner";
    const string& SetplayParserSExpr::cond_playmode_element = "playm";
    const string& SetplayParserSExpr::cond_and_element = "and";
    const string& SetplayParserSExpr::cond_or_element = "or";
    const string& SetplayParserSExpr::cond_not_element = "not";


    // Initialize map of strings to playmode
    const std::map<std::string, PlayMode> SetplayParserSExpr::str_to_playmode = {
        std::pair<std::string, PlayMode>("bko", BeforeKickOff),
        std::pair<std::string, PlayMode>("time_over", TimeOver),
        std::pair<std::string, PlayMode>("play_on", PlayOn),
        std::pair<std::string, PlayMode>("ko_our", KickOff_Our),
        std::pair<std::string, PlayMode>("ko_opp", KickOff_Opp),
        std::pair<std::string, PlayMode>("fk_our", FreeKick_Our),
        std::pair<std::string, PlayMode>("fk_opp", FreeKick_Opp),
        std::pair<std::string, PlayMode>("ifk_our", Ind_FreeKick_Our),
        std::pair<std::string, PlayMode>("ifk_opp", Ind_FreeKick_Opp),
        std::pair<std::string, PlayMode>("ck_our", CornerKick_Our),
        std::pair<std::string, PlayMode>("ck_opp", CornerKick_Opp),
        std::pair<std::string, PlayMode>("gk_our", GoalKick_Our),
        std::pair<std::string, PlayMode>("gk_opp", GoalKick_Opp),
        std::pair<std::string, PlayMode>("gc_our", GoalieCatch_Our),
        std::pair<std::string, PlayMode>("gc_opp", GoalieCatch_Opp),
        std::pair<std::string, PlayMode>("ag_our", AfterGoal_Our),
        std::pair<std::string, PlayMode>("ag_opp", AfterGoal_Opp)
    };

    SetplayParserSExpr::SetplayParserSExpr()
    {

    }

    vector<Setplay>* SetplayParserSExpr::parse(const string filePath) {

    }

    vector<Setplay>* SetplayParserSExpr::parse(const char* in) {
        return parseSetplay(in, out);
    }


    int SetplayParserSExpr::parseSetplay(const char* in, Setplay** out) {
        int start_offset = in;

        // Parse set-play start element
        if (parseStartElementSymbol(in, setplay_element) == 0) {
            std::cerr << "Setplay start element symbol not found" << std::endl;
            return in - start_offset;
        }


        // Get setplay abort time (or abort condition) if any => TODO Must parse ints or a condition. If an int is parsed, create an CondWaitTime (do the same for wait time)
        ICondition* abort_condition = &CondBool::False;
        if (parseAttribute(abort_time_attribute) == 0) {
            std::cerr << "Setplay abort condition not found (not mandatory?)" << std::endl;
            return in - start_offset;
        } else {
            // Try to parse the condition
            if (parseCondition(in, &abort_condition) == 0) {
                std::cerr << "Setplay abort condition value not found (not mandatory?)" << std::endl;
            }
        }

        // Get setplay wait time (or abort condition) if any => TODO Must parse ints or a condition. If an int is parsed
        ICondition* wait_condition = &CondBool::False;
        if (parseAttribute(wait_time_attribute) == 0) {
            std::cerr << "Setplay wait  condition not found (not mandatory?)" << std::endl;
            return in - start_offset;
        } else {
            // Try to parse the condition
            if (parseCondition(in, &wait_condition) == 0) {
                std::cerr << "Setplay wait condition value not found (not mandatory?)" << std::endl;
            }
        }

        // Get the participations of the step
        string setplay_name = "";
        if (parseAttribute(participations_attribute) == 0) {
            std::cerr << "Setplay name attribute not found" << std::endl;
        } else {
            if (parseString(in, setplay_name) == 0) {
                std::cerr << "Setplay name value not found" << std::endl;
            }
        }

        // Get the id of the setplay
        int setplay_id;
        if (parseAttribute(id_attribute) == 0) {
            std::cerr << "Setplay id attribute not found" << std::endl;
        } else {
            if (parseInt(in, setplay_id) == 0) {
                std::cerr << "Setplay id value not found" << std::endl;
                return in - start_offset;
            }
        }

        // Get invertability of the setplay
        bool invertable = false;
        if (parseAttribute(invertable_attribute) == 0) {
            std::cerr << "Setplay invertable attribute not found" << std::endl;
        } else {
            if (parseBool(in, invertable) == 0) {
                std::cerr << "Setplay invertable value not found" << std::endl;
                return in - start_offset;
            }
        }

        // Parse steps
        vector<Step*> steps = new vector<Step*>();
        if (parseAttribute(steps_attribute) == 0) {
            std::cerr << "Setplay steps attribute not found" << std::endl;
        } else {
            do {
                Step* step = NULL;
                if (parseStep(in, &step) == 0 && steps.empty()) { // Should be &step
                    std::cerr << "No step found " << std::endl;
                    delete steps;
                    return in - start_offset;
                } else {
                    steps.push_back(step);
                }
            } while (step!= NULL);
        }

        // Parser setplay end element symbol
        if (parseEndElementSymbol(in) == 0) {
            std::cerr << "Setplay end element not found" << std::endl;
            return in - start_offset;
        }

        // Create the parsed setplay
        *setplay = new Setplay(setplay_id, setplay_name, steps, invertable, abort_condition);
    }

    /*
     * Elements parsing
     */
    int SetplayParserSExpr::parseStep(const char* in, Step** step) {
        int start_offset = in;

        // Parse Step start element
        if (parseStartElementSymbol(in, step_element) == 0) {
            std::cerr << "Step start element symbol not found" << std::endl;
            return in - start_offset;
        }

        // Get the id of the step
        unsigned int step_id;
        if (parseAttribute(id_attribute) == 0) {
            std::cerr << "Step id attribute not found" << std::endl;
        } else {
            if (parseInt(in, &step_id) == 0) {
                std::cerr << "Step id value not found" << std::endl;
                return in - start_offset;
            }
        }

        // Get step abort condition if any
        ICondition* abort_condition = &CondBool::False;
        if (parseAttribute(abort_condition_attribute) == 0) {
            std::cerr << "Setplay abort condition not found (not mandatory?)" << std::endl;
            return in - start_offset;
        } else {
            // Try to parse the condition
            if (parseCondition(in, &abort_condition) == 0) {
                std::cerr << "Setplay abort condition value not found (not mandatory?)" << std::endl;
            }
        }

        // Get step wait condition if any
        ICondition* wait_condition = &CondBool::False;
        if (parseAttribute(wait_condition_attribute) == 0) {
            std::cerr << "Setplay abort condition not found (not mandatory?)" << std::endl;
            return in - start_offset;
        } else {
            // Try to parse the condition
            if (parseCondition(in, &wait_condition) == 0) {
                std::cerr << "Setplay abort condition value not found (not mandatory?)" << std::endl;
            }
        }

        // Get step wait condition if any
        ICondition* wait_condition = &CondBool::False;
        if (parseAttribute(wait_condition_attribute) == 0) {
            std::cerr << "Step abort condition not found (not mandatory?)" << std::endl;
            return in - start_offset;
        } else {
            // Try to parse the condition
            if (parseCondition(in, &wait_condition) == 0) {
                std::cerr << "Setplay abort condition value not found (not mandatory?)" << std::endl;
            }
        }

        // Parse participations
        vector<Participation*>* participations = new vector<Participation*>();
        if (parseAttribute(participations_attribute) == 0) {
            std::cerr << "Setplay participations attribute not found" << std::endl;
        } else {
            do {
                Participation* participation = NULL;
                if (parseParticipation(in, &participation) == 0 && participations.empty()) { // Should be &step
                    std::cerr << "No participations found " << std::endl;
                    delete participations;
                    return in - start_offset;
                } else {
                    participations.push_back(participation);
                }
            } while (participation != NULL);
        }

        // Get step start condition if any
        ICondition* start_condition = &CondBool::True;
        if (parseAttribute(start_condition_attribute) == 0) {
            std::cerr << "Step start condition not found (not mandatory?)" << std::endl;
            return in - start_offset;
        } else {
            // Try to parse the condition
            if (parseCondition(in, &start_condition) == 0) {
                std::cerr << "Setplay start condition value not found (not mandatory?)" << std::endl;
            }
        }

        // Get step transitions
        vector<Transition*>* transitions = new vector<Transition>();
        if (parseAttribute(transitions_attribute) == 0) {
            std::cerr << "Step transitions attribute not found (not mandatory?)" << std::endl;
            return in - start_offset;
        } else {
            do {
                // TODO Check if there is a list element!!
                Transition* transition = NULL;
                if (parseParticipation(in, &transition) == 0 && transitions.empty()) { // Should be &step
                    std::cerr << "No transitions found " << std::endl;
                    delete transitions;
                    return in - start_offset;
                } else {
                    transitions.push_back(transition);
                }
            } while (transition != NULL);
        }


        // Get step leader
        IPlayer* leader = NULL;
        if (parseAttribute(leader_attribute) == 0) {
            std::cerr << "Step leader attribute not found (not mandatory?)" << std::endl;
            return in - start_offset;
        } else {
            // Try to parse player
            if (parsePlayer(in, &leader) == 0) {
                std::cerr << "Setplay leader value not found (not mandatory?)" << std::endl;
            }
        }

        // Parse step end element symbol
        if (parseEndElementSymbol(in) == 0) {
            std::cerr << "Step end element not found" << std::endl;
            return in - start_offset;
        }

        *step = new Step(step_id,
                         leader,
                         transitions,
                         participations,
                         wait_condition,
                         abort_condition,
                         start_condition);
        return  in - start_offset;
    }

    int SetplayParserSExpr::parseParticipation(const char* in, Participation** participation) {
        int start_offset = in;

        // Parse Step start element
        if (parseStartElementSymbol(in, participation_element) == 0) {
            std::cerr << "Participation start element symbol not found" << std::endl;
            return in - start_offset;
        }

        // Parse the player definition
        IPlayer* participant = NULL;
        if (parsePlayer(in, &participant) == 0) {
            std::cerr << "No participant found" << std::endl;
            return in - start_point;
        }

        // Parse the location
        ILocation* location = NULL;
        if (! (parseLocation(in, &location) == 0)) {
            std::cerr << "No location found" << std::endl;
            return in - start_point;
        }

        // Parse participation end element symbol
        if (parseEndElementSymbol(in) == 0) {
            std::cerr << "Participation end element not found" << std::endl;
            return in - start_offset;
        }

        *participation = new Participation(participant, location);
        return in - start-offset;
    }

    int SetplayParserSExpr::parseTransition(const char* in, Transition** transition) {
        int start_point = in;
        NextStep* next_step = NULL;
        if (parseNextStep(in, &next_step) != 0) {
            transition = next_step;
            return in - start_point;
        }

        Abort* abort = NULL;
        if (parseAbort(in, &abort) != 0) {
            transition = abort;
            return in - start_point;
        }

        Finish* finish = NULL;
        if (parseFinish(in, &finish) != 0) {
            transition = finish;
            return in - start_point;
        }

        std::cerr << "No transition found" << std::endl;
    }

    int SetplayParserSExpr::parseNextStep(const char* in, NextStep** next_step) {
        int start_point = in;
        if (parseStartElementSymbol(in, next_step_element) == 0) {
            return in - start_point;
        }

        // Parse next step id attribute
        if (parseAttribute(next_step_id_attribute) == 0) {
            std::cerr << "Next step id attribute not found" << std::endl;
            return in - start_point;
        }

        // Parse next step id value
        unsigned int next_step_id;
        // Parse next step id attribute
        if (parseUInt(in, next_step_id) == 0) {
            std::cerr << "Next step id value not found" << std::endl;
            return in - start_point;
        }

        // Parse start condition attribute
        if (parseAttribute(start_condition_attribute) == 0) {
            std::cerr << "Next step start condition attribute not found" << std::endl;
            return in - start_point;
        }
        ICondition* start_condition = &CondBool::True;
        if (parseCondition(in, start_condition) == 0) {
            std::cerr << "Next step start condition value not found" << std::endl;
        }


        // Parse directives attribute
        vector<Directive*>* directives = NULL;
        if (parseAttribute(directives_attribute) == 0) {
            std::cerr << "Next step directives attribute not found" << std::endl;
            // return in - start_point;
        } else {
            directives = new vector<Directive*>();
            do {
                Directive* directive = NULL;
                if (parseDirective(in, &directive) == 0 && directives->empty()) {
                    std::cerr << "Directives values not found" << std::endl;
                    delete directives;
                } else {
                    directives->push_back(directive);
                }
            } while (directive != NULL);
        }

        // Parse end element symbol
        if (parseEndElementSymbol(in)) {
            std::cerr << "Next step end element not found" << std::endl;
            return in - start_point;
        }

        *next_step = new NextStep(next_step_id, start_condition, directives);
        return in - start_point;
    }

    int SetplayParserSExpr::parseAbort(const char* in, Abort** abort) {
        int start_point = in;
        if (parseStartElementSymbol(in, abort_transition_element) == 0) {
            return in - start_point;
        }

        // Parse start condition attribute
        if (parseAttribute(start_condition_attribute) == 0) {
            std::cerr << "Abort transition start condition attribute not found" << std::endl;
            return in - start_point;
        }
        ICondition* start_condition = &CondBool::True;
        if (parseCondition(in, start_condition) == 0) {
            std::cerr << "Abort transition start condition value not found" << std::endl;
        }

        // Parse directives attribute
        vector<Directive*>* directives = NULL;
        if (parseAttribute(directives_attribute) == 0) {
            std::cerr << "Abort transition directives attribute not found" << std::endl;
            // return in - start_point;
        } else {
            directives = new vector<Directive*>();
            do {
                Directive* directive = NULL;
                if (parseDirective(in, &directive) == 0 && directives->empty()) {
                    std::cerr << "Abort transition directives values not found" << std::endl;
                    delete directives;
                } else {
                    directives->push_back(directive);
                }
            } while (directive != NULL);
        }

        // Parse end element symbol
        if (parseEndElementSymbol(in)) {
            std::cerr << "Abort end element not found" << std::endl;
            return in - start_point;
        }

        *abort = new Abort(start_condition, directives);
        return in - start_point;
    }

    int SetplayParserSExpr::parseFinish(const char* in, Finish** finish) {
        int start_point = in;
        if (parseStartElementSymbol(in, finish_transition_element) == 0) {
            return in - start_point;
        }

        // Parse start condition attribute
        if (parseAttribute(start_condition_attribute) == 0) {
            std::cerr << "Finish transition start condition attribute not found" << std::endl;
            return in - start_point;
        }
        ICondition* start_condition = &CondBool::True;
        if (parseCondition(in, start_condition) == 0) {
            std::cerr << "Finish transition start condition value not found" << std::endl;
        }

        // Parse directives attribute
        vector<Directive*>* directives = NULL;
        if (parseAttribute(directives_attribute) == 0) {
            std::cerr << "Finish directives attribute not found" << std::endl;
            // return in - start_point;
        } else {
            directives = new vector<Directive*>();
            do {
                Directive* directive = NULL;
                if (parseDirective(in, &directive) == 0 && directives->empty()) {
                    std::cerr << "Finish transition directives values not found" << std::endl;
                    delete directives;
                } else {
                    directives->push_back(directive);
                }
            } while (directive != NULL);
        }

        // Parse end element symbol
        if (parseEndElementSymbol(in)) {
            std::cerr << "Abort end element not found" << std::endl;
            return in - start_point;
        }

        *finish = new Finish(start_condition, directives);
        return in - start_point;
    }

    int SetplayParserSExpr::parseDirective(const char* in, Directive** directive) {
        // HACK Parse only do directives => They're the ones being used
        int start_point = in;

        // Parse start of do directive
        if (parseStartElementSymbol(in, directive_do) == 0) {
            std::cerr << "No directive found" << std::endl;
            return in - start_point;
        }

        // Parse player executing directive => In old version several players can do this => Create N directives?
        IPlayer* executor = NULL;
        if (parseAttribute(in, players_attribute) == 0) {
            std::cerr << "Directive players attribute not found" << std::endl;
            return in - start_point;
        }
        if (parseAction(in, &executor) == 0) {
            std::cerr << "Directive player value found" << std::endl;
            return in - start_point;
        }

        // Parse the actions of the directive
        vector<IAction*>* actions = NULL;
        if (parseAttribute(in, actions_attribute) == 0) {
            std::cerr << "No actions found" << std::endl;
        } else {
            actions = new vector<IAction*>();
            do {
                IAction* action = NULL;
                if (parseAction(in, &action) == 0) {
                    if (actions->empty()) {
                        delete actions;
                        return in - start_point;
                    }
                    break;
                }
                actions->push_back(action);
            } while (action != NULL);
        }

        // Parse end element
        if (parseEndElementSymbol(in) == 0) {
            std::cerr << "Directive end element not found" << std::endl;
        }

        // TODO Check how many players are there and create a directive for each => Never more than one was used!!!!
        *directive = new Directive(executor, actions);
        return in - start_point;
    }

    /**
      * Parsing of players
      */
    int SetplayParserSExpr::parsePlayer(const char* in, IPlayer* out) {
        int start_point = in;

        if (parseStartElementSymbol(in, finish_transition_element) == 0) {
            return in - start_point;
        }

        // Parse end element
        if (parseEndElementSymbol(in) == 0) {
            std::cerr << "Directive end element not found" << std::endl;
        }
    }

    int SetplayParserSExpr::parsePlayerID(const char* in, PlayerID* out) {

        if (parseStartElementSymbol(in, player_element)) {

        }

        // Parse end element
        if (parseEndElementSymbol(in) == 0) {
            std::cerr << "Player ID end element not found" << std::endl;
        }
    }

    int SetplayParserSExpr::parsePlayerVar(const char* in, PlayerVar* out) {
        // Former player role

    }

    int SetplayParserSExpr::parseAction(const char* in, IActionSoccer* out) {

    }

    int SetplayParserSExpr::parseRegion(const char* in, IRegion2D** out) {

    }

    int SetplayParserSExpr::parsePoint(const char* in, IPoint2D** out) {

    }

    int SetplayParserSExpr::parseCondition(const char* in, ICondition** out) {
    }

    int SetplayParserSExpr::parseVariable(const char* in, SetplayVar** out) {

    }

    /*
     * Auxiliary functions
     */


    int SetplayParserSExpr::parseStartElementSymbol(const char* in, string const& element_name) {
      rule<> sp_p = *space_p >> "(" >> as_lower_d[str_p(element_name)];

      parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in.c_str(),sp_p,nothing_p);

      if(!result.hit) {
          return 0;
      }

      in += result.length;
      return result.length;
    }

    int SetplayParserSExpr::parseAttribute(const string& attribute_name,
                                           const char* in) {
        rule<> sp_p = *space_p >> ":" >> as_lower_d[attribute_name];

      parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in.c_str(),sp_p,nothing_p);

      if(!result.hit) {
          return 0;
      }

      in += result.length;
      return result.length;
    }

    int SetplayParserSExpr::parseEndElementSymbol(const char* in) {
      rule<> sp_p = *space_p >> ")";

      parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in.c_str(),sp_p,nothing_p);

      if(!result.hit) {
          return 0;
      }
      in += result.length;
      return result.length;
    }

    int SetplayParserSExpr::parseInt(const char* in, int& value) {
      rule<> sp_p = *space_p >> int_p[assign_a(value)];

      parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in,sp_p,nothing_p);

      if(!result.hit) {
          return 0;
      }
      in += result.length;
      return result.length;
    }

    int SetplayParserSExpr::parseUint(const char* in, unsigned int& value) {
      rule<> sp_p = *space_p >> uint_p[assign_a(value)];

      parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in,sp_p,nothing_p);

      if(!result.hit) {
          return 0;
      }
      in += result.length;
      return result.length;
    }


    int SetplayParserSExpr::parseDouble(const char* in, double& value) {
      rule<> sp_p = *space_p >> real_p[assign_a(value)];

      parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in,sp_p,nothing_p);

      if(!result.hit) {
          return 0;
      }
      in += result.length;
      return result.length;
    }

    int SetplayParserSExpr::parseString(const char* in, string& value) {
      rule<> sp_p = *space_p >> str_p[assign_a(value)];

      parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in,sp_p,nothing_p);

      if(!result.hit) {
          return 0;
      }
      in += result.length;
      return result.length;
    }

    int SetplayParserSExpr::parseBool(const char* in, bool& value) {
      rule<> sp_p = *space_p >> as_lower_d["true"];

      parse_info<> result = BOOST_SPIRIT_CLASSIC_NS::parse(in,sp_p,nothing_p);

      if(!result.hit) {
          value = false;
          return 0;
      }
      value = true;
      in += result.length;
      return result.length;
    }

}
