#include "setplay_parser_sexpr_driver.h"

#include <fstream>
#include <sstream>
#include <setplay/parsers/sexpr/setplay_scanner_sexpr.h>

#include <cassert>

namespace setplay {
namespace parsers {

SetplayParserSexprDriver::SetplayParserSexprDriver(bool trace_scanning,
                                                   bool trace_parsing)
    : M_trace_scanning(trace_scanning)
    , M_trace_parsing(trace_parsing)
    , M_state(OTHER)
{

}

SetplayParserSexprDriver::~SetplayParserSexprDriver()
{

}

SetplayScannerSexpr* SetplayParserSexprDriver::lexer() const
{
    return M_lexer;
}

bool SetplayParserSexprDriver::parseStream(std::istream& in)
{
    SetplayScannerSexpr scanner(&in);
    scanner.setDebug(M_trace_scanning);

    this->M_lexer = &scanner;
    SetplayParserSexpr parser(*this);
    parser.set_debug_level(M_trace_parsing);

    while ( parser.parse() == 0 );
    this->M_lexer = NULL;

    return M_setplays.size() > 0;
}

bool SetplayParserSexprDriver::parseFile(const std::string &filename)
{
    M_file = filename;
    std::ifstream in(filename.c_str());
    if (!in.good()) return false;

    return parseStream(in);
}

bool SetplayParserSexprDriver::parseString(const std::string &input)
{
    std::istringstream iss(input);
    return parseStream(iss);
}

std::string& SetplayParserSexprDriver::file()
{
    return M_file;
}

void SetplayParserSexprDriver::setState(State state)
{
    M_state = state;
}

SetplayParserSexprDriver::State SetplayParserSexprDriver::state() const
{
    return M_state;
}

void SetplayParserSexprDriver::add(Setplay* setplay)
{
    M_setplays.push_back(setplay);
}

vector<Setplay*>& SetplayParserSexprDriver::setplays()
{
    return M_setplays;
}

void
SetplayParserSexprDriver::error(const location& l, const std::string& m)
{
    std::cerr << "Error " << m << " at Location " << l << std::endl;
}

void
SetplayParserSexprDriver::error(const std::string& m)
{
    std::cerr << m << std::endl;
    std::flush(std::cerr);
}

void SetplayParserSexprDriver::log(const std::string& m)
{
	std::cerr << m << std::endl;
}

}
}
