// $Id: scanner.h 17 2007-08-19 18:51:39Z tb $

#ifndef SP_SETPLAY_SCANNER_SEXPR_H
#define SP_SETPLAY_SCANNER_SEXPR_H

// Flex expects the signature of yylex to be defined in the macro YY_DECL, and
// the C++ parser expects it to be declared. We can factor both as follows.

#ifndef YY_DECL

#define	YY_DECL						\
    setplay::parsers::SetplayParserSexpr::token_type				\
    setplay::parsers::SetplayScannerSexpr::lex(				\
    setplay::parsers::SetplayParserSexpr::semantic_type* yylval,		\
    setplay::parsers::SetplayParserSexpr::location_type* yylloc,		\
    setplay::parsers::SetplayParserSexprDriver& driver		\
    )
#endif

#ifndef __FLEX_LEXER_H
#define yyFlexLexer ExampleFlexLexer
#include "FlexLexer.h"
#undef yyFlexLexer
#endif

#include "setplay_parser_sexpr.hpp"
#include "setplay_parser_sexpr_driver.h"

namespace setplay {
namespace parsers {

/** Scanner is a derived class to add some extra function to the scanner
 * class. Flex itself creates a class named yyFlexLexer, which is renamed using
 * macros to ExampleFlexLexer. However we change the context of the generated
 * yylex() function to be contained within the Scanner class. This is required
 * because the yylex() defined in ExampleFlexLexer has no parameters. */
class SetplayScannerSexpr
        : public ExampleFlexLexer
{
public:
    /** Create a new scanner object. The streams arg_yyin and arg_yyout default
     * to cin and cout, but that assignment is only made when initializing in
     * yylex(). */
    SetplayScannerSexpr(std::istream* arg_yyin = 0,
            std::ostream* arg_yyout = 0);

    /** Required for virtual functions */
    virtual ~SetplayScannerSexpr();

    /** This is the main lexing function. It is generated by flex according to
     * the macro declaration YY_DECL above. The generated bison parser then
     * calls this virtual function to fetch new tokens. */
    virtual SetplayParserSexpr::token_type lex( SetplayParserSexpr::semantic_type* yylval,
                                                SetplayParserSexpr::location_type* yylloc,
                                                SetplayParserSexprDriver& driver);

    /** Enable debug output (via arg_yyout) if compiled into the scanner. */
    void setDebug(bool b);
};

}
}

#endif
