// -*-c++-*-

/***************************************************************************
                                IActionExecutor.h
                       Abstract base class for CLang Actions
                             -------------------
    begin                : 28-MAY-2002
    copyright            : (C) 2006 Luis Mota
    email                : luis.mota@iscte.pt
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU LGPL as published by the Free Software  *
 *   Foundation; either version 2 of the License, or (at your option) any  *
 *   later version.                                                        *
 *                                                                         *
 ***************************************************************************/
#ifndef SP_ACTION_EXECUTOR_H
#define SP_ACTION_EXECUTOR_H

#include <rcsc/geom.h>

#include <setplay/model/participants/player_affiliation.h>

namespace setplay {

     class IActionExecutor {
        public:
            virtual ~IActionExecutor() { }

            /// True if the action was executed
            virtual bool noAction() = 0;

            virtual bool moveTo( const rcsc::Vector2D& position ) = 0;
            virtual bool moveTo( const rcsc::Region2D& region ) = 0;

            virtual bool kickTo( const rcsc::Vector2D& position ) = 0;

            virtual bool passToPlayer( const PlayerAffiliation& player ) = 0;
            virtual bool passToPoint( rcsc::Vector2D point ) = 0;
            virtual bool passToRegion( rcsc::Region2D& region ) = 0;
            virtual bool forwardToPlayer( const PlayerAffiliation& player ) = 0;

            virtual bool shoot( ) = 0;

            virtual bool dribbleTo( const rcsc::Vector2D& position ) = 0;
            virtual bool dribbleTo( const rcsc::Region2D& region ) = 0;
            virtual bool holdBall() = 0;

            virtual bool waitAtPosition( const rcsc::Vector2D& positions ) = 0;
            virtual bool waitAtRegion( const rcsc::Vector2D& positions ) = 0;

            virtual bool openPassLineForPlayer( const PlayerAffiliation& player ) = 0;

            // Interception
            virtual bool interceptPlayer( const PlayerAffiliation& player ) = 0;
            virtual bool interceptBall( ) = 0;
            virtual bool interceptBallAt( rcsc::Vector2D point ) = 0;

            virtual bool followPlayer( const PlayerAffiliation& player,
                                       const rcsc::Vector2D& offset = rcsc::Vector2D(0,0) ) = 0;
            virtual bool followBall( rcsc::Vector2D offset = rcsc::Vector2D(0,0) ) = 0;

            virtual bool markPlayer( const PlayerAffiliation& player ) = 0;

             /// Look at commands
            virtual bool lookAt( const rcsc::Vector2D& position ) = 0;
            virtual bool lookAt( const rcsc::Region2D& region ) = 0;
            virtual bool lookAt( const PlayerAffiliation& player ) = 0;
            virtual bool lookAt( const std::vector<PlayerAffiliation>& players ) = 0;
    };

}

#endif
