#ifndef SETPLAY_COMMUNICATION_H
#define SETPLAY_COMMUNICATION_H


// For messages processing
#include <setplay/communication/messages.h>
#include <setplay/runtime/setplay_interpreter.h>

namespace setplay {
    class SetplayDatabase;

    class MessageBox
    {
    public:
            typedef std::vector<IMessage*> Messages;

            MessageBox();
            ~MessageBox();

            void pushMessage(IMessage* message);
            Messages* messages();
            unsigned int numMessages() const;

            void reset();

            void print(std::ostream& os) const;
        private:
            Messages M_messages;
    };

    class SetplayCommunication
    {
    public:
        SetplayCommunication();
        ~SetplayCommunication();

        /* COMMUNICATION
         * Handle received messages
         * Distribute messages to send
         */

        void processOutbox();

        MessageBox* outbox();
        MessageBox* inbox();
        void processInbox();
        void receive(IMessage* message);

        void setInterpreter( SetplayInterpreter* interpreter )
        {
            M_interpreter = interpreter;
        }

        void setDatabase( const SetplayDatabase* database )
        {
            M_database = database;
        }

        void reset();

    private:
        unsigned int M_last_inboung_msg_time;
        unsigned int M_last_outbound_msg_time;
        SetplayInterpreter::ExecutionState M_last_execution_state;

        class TimedMessage
        {
        public:
            TimedMessage( IMessage* message, unsigned int time_to_live = 0);

            const IMessage* getMessage();
            unsigned int timeToLive() const;

        private:
            IMessage* M_message;
            unsigned int M_time_to_live;

        };

        void log(const std::string& message) const;

        MessageBox M_inbox;
        MessageBox M_outbox;

        bool hasMessagesToSend();
        IMessage* messageToSend();
        // 14/06/11 FA - Should a message be sent? This should be handled in a upper-level class
        bool repeatMessage() const;
        bool willComunicateInitMsg();
        bool willComunicateStepMsg();


        // Accessors
        bool commAllowed() const;
        bool singleChannelComm() const;
        unsigned int messageRepeatTime() const;

        /// Communication handling
        bool shouldProcessMessage(const IMessage& message ) const;
        void process(const SetplayStartMessage& message);
        void process(const StepEnterMessage& msg);
        void process(const StepTransitionMessage& message);
        void process(const ActionDoneMessage& message);
        void process(const SetplayFinishMessage& message);
        void process(const SetplayAbortMessage& message);

        /// Setplay State Machine logic
        bool shouldChangeSetplay(const SetplayStartMessage& message) const;
        bool shouldRestartSetplay(const SetplayStartMessage& message) const;
        bool shouldChangeStep(const StepTransitionMessage &message) const;
        bool shouldRestartStep(const StepTransitionMessage &message) const;

        SetplayInterpreter* M_interpreter;
        const SetplayDatabase* M_database;
    };

}

#endif // SETPLAY_COMMUNICATION_H
