#include "setplay_mediator.h"

#include <setplay/runtime/setplay_selector.h>
#include <setplay/runtime/player_selector.h>
#include <setplay/runtime/transition_selector.h>
#include <setplay/runtime/iaction_executor.h>
#include <setplay/runtime/icondition_evaluator.h>

namespace setplay {

SetplayMediator::SetplayMediator()
    : M_global_space("globals")
    , M_condition_evaluator( NULL )
    , M_action_executor( NULL )
{
    FirstFeasibleSetplaySelector* setplay_selector = new FirstFeasibleSetplaySelector();
    // setplay_selector->setPlayerSelector( new PlayersMinGlobalDistanceSelector( M_world_model ) );
    M_setplay_selector = setplay_selector;
    M_setplay_interpreter.setTransitionSelector( new FirstFeasibleTransitionSelector() );
}

SetplayMediator::~SetplayMediator()
{
    if ( M_setplay_selector != NULL )
    {
        delete M_setplay_selector;
    }

    if ( M_action_executor!= NULL )
    {
        delete M_action_executor;
    }

    if ( M_condition_evaluator != NULL )
    {
        delete M_condition_evaluator;
    }

}

void SetplayMediator::update()
{
    SetplayInterpreter::ExecutionState last_state = M_setplay_interpreter.executionState();


    if ( !M_setplay_interpreter.hasActiveSetplay() )
    {
        M_setplay_selector->selectSetplay( M_setplay_interpreter );
        if ( !M_setplay_interpreter.hasActiveSetplay() )
        {
            return;
        }
    }

    M_setplay_interpreter.update();
}

void SetplayMediator::setSetplaySelector(ISetplaySelector* setplay_selector)
{
    assert( setplay_selector != NULL );
    if ( M_setplay_selector != NULL ) { delete M_setplay_selector; }
    M_setplay_selector = setplay_selector;
}

void SetplayMediator::setConditionEvaluator(IConditionEvaluator* condition_evaluator)
{
    assert( condition_evaluator != NULL );
    if ( M_condition_evaluator != NULL ) { delete M_condition_evaluator; }
    M_condition_evaluator = condition_evaluator;
}

void SetplayMediator::setActionExecutor(IActionExecutor* action_executor)
{
    assert( action_executor != NULL );
    if ( M_action_executor != NULL ) { delete M_action_executor; }
    M_action_executor = action_executor;
}


/// Error handling
void SetplayMediator::log(std::string message)
{
    std::cerr << message << std::endl;
}

}
