#ifndef SETPLAY_MEDIATOR_H
#define SETPLAY_MEDIATOR_H

#include <setplay/runtime/memory_space.h>
#include <setplay/runtime/setplay_interpreter.h>
#include <setplay/runtime/setplay_database.h>

#include <setplay/runtime/setplay_selector.h>
#include <setplay/runtime/player_selector.h>
#include <setplay/runtime/transition_selector.h>

#include <setplay/runtime/setplay_communication.h>
#include <setplay/communication/messages.h>

namespace setplay {

class SetplayMediator
{
public:
    SetplayMediator();
    ~SetplayMediator();

    virtual void update();

    // Accessors
    // IPlayerSelector& playerSelector() const;


    MemorySpace& globalSpace()
    {
        return M_global_space;
    }

    const SetplayDatabase& setplayDatabase() const
    {
        return M_setplay_database;
    }

    SetplayInterpreter* getInterpreter(unsigned int )
    {
        return &M_setplay_interpreter;
    }

    const IWorldModel* worldModel()
    {
        return M_world_model;
    }

    void setSetplaySelector(ISetplaySelector* setplay_selector);
    void setConditionEvaluator(IConditionEvaluator* condition_evaluator);
    void setActionExecutor(IActionExecutor* action_executor);

    void receive(IMessage* message);

protected:
    /// Error handling
    void log(std::string message);

    MemorySpace M_global_space;
    SetplayInterpreter M_setplay_interpreter;
    SetplayDatabase M_setplay_database;

    ISetplaySelector* M_setplay_selector;
    IConditionEvaluator* M_condition_evaluator;
    IActionExecutor* M_action_executor;
    IWorldModel* M_world_model;

    setplay::PlayerAffiliation* M_self;

    /// TODO General configurations
    bool M_derive_wait_conditions_from_step_participations;
};

}
#endif // SETPLAY_MEDIATOR_H
