#ifndef EVENT_PARTICIPANTS_H
#define EVENT_PARTICIPANTS_H

#include <map>
#include <vector>
#include <soccer/events/ievent_visitor.h>

namespace soccer {
namespace event {

/*!
  \brief Given a sequence of events, indexed them by player participation and sort them ascending by time
  */

class PlayersInEventsExtractor
        : public soccer::event::IEventVisitor
{
public:
    PlayersInEventsExtractor();
    const std::map<PlayerId, std::vector<const IEvent *> > &playerEvents() const;
    const std::vector<soccer::PlayerId>& players() const;

    void visit(const KickOff& event);
    void visit(const KickOffTaken& event);
    void visit(const ThrowIn& event);
    void visit(const ThrowInTaken& event);
    void visit(const GoalKick& event);
    void visit(const GoalKickTaken& event);
    void visit(const CornerKick& event);
    void visit(const CornerKickTaken& event);
    void visit(const FreeKick& event);
    void visit(const FreeKickTaken& event);
    void visit(const IndirectFreeKick& event);
    void visit(const IndirectFreeKickTaken& event);

    void visit(const Goal& event);

    void visit(const SoccerMomentChanged& event);
    void visit(const PossessionChanged& event);

    void visit(const Shoot& event);
    void visit(const Dribble& event);
    void visit(const Pass& event);

    void visit(const Interception& event);
    void visit(const Turnover& event);

    void visit(const PlayerMove& event);

    void visit(const GoalieCatch& event);
    void visit(const BallKick& event);
    void visit(const BallDispute& event);
    void visit(const BallMove& event);
    void visit(const BallHold& event);

    void visit(const Offside& event);

    void visit(const PassChain& event);
    void visit(const GoalPassChain& event);
private:
    void addParticipants( const EventChain& event );
    void addParticipant( PlayerId player_id, const IEvent &event  );

    /// Events and pointers should be const
    std::map<soccer::PlayerId, std::vector< const IEvent* > > M_player_events;
    std::vector<soccer::PlayerId > M_players;
};

}
}

#endif // EVENT_PARTICIPANTS_H
