#include <soccer/events/ievent.h>

using namespace std;

namespace soccer {
namespace event {

unsigned int IEvent::duration() const
{
    return endTime() - startTime();
}


map<IEvent::EventType, string> createEventTypeToStringMap()
{
    map<IEvent::EventType, string> mmap;

    mmap.insert(pair<IEvent::EventType, string>(IEvent::ThrowIn, "throw-in"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::ThrowInTaken, "throw-in-taken" ));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::KickOff, "kick-off"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::KickOffTaken, "kick-off-taken"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::CornerKick, "corner-kick"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::CornerKickTaken, "corner-kick-taken"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::FreeKick, "free-kick"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::FreeKickTaken, "free-kick-taken"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::IndirectFreeKick,"indirect-free-kick"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::IndirectFreeKickTaken, "indirect-free-kick-taken"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::GoalKick,"goal-kick"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::GoalKickTaken,"goal-kick-taken"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::GoalOpportunity,"goal-opportunity"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::Offside, "offside"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::Dribble, "dribble"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::Pass, "pass"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::Shoot, "shoot"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::Goal, "goal"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::BallDispute, "ball-dispute"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::BallKick, "ball-kick"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::BallHold, "ball-hold"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::GoalieCatch, "goalie-catch"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::Interception, "interception"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::BallMove, "ball-move"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::PlayerMove, "player-move"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::PossessionChanged, "possession-changed"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::Turnover, "turnover"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::SoccerMomentChanged, "soccer-moment-changed"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::PassChain, "pass-chain"));
    mmap.insert(pair<IEvent::EventType, string>(IEvent::GoalPassChain, "goal-pass-chain"));

    return mmap;
}

map<string, IEvent::EventType> createStringToEventTypeMap()
{
    map<string, IEvent::EventType> mmap;

    mmap.insert(pair<string, IEvent::EventType>("throw-in", IEvent::ThrowIn));
    mmap.insert(pair<string, IEvent::EventType>("throw-in-taken", IEvent::ThrowInTaken ));
    mmap.insert(pair<string, IEvent::EventType>("kick-off", IEvent::KickOff));
    mmap.insert(pair<string, IEvent::EventType>("kick-off-taken", IEvent::KickOffTaken));
    mmap.insert(pair<string, IEvent::EventType>("corner-kick", IEvent::CornerKick));
    mmap.insert(pair<string, IEvent::EventType>("corner-kick-taken", IEvent::CornerKickTaken));
    mmap.insert(pair<string, IEvent::EventType>("free-kick", IEvent::FreeKick));
    mmap.insert(pair<string, IEvent::EventType>("free-kick-taken", IEvent::FreeKickTaken));
    mmap.insert(pair<string, IEvent::EventType>("indirect-free-kick", IEvent::IndirectFreeKick));
    mmap.insert(pair<string, IEvent::EventType>("indirect-free-kick-taken", IEvent::IndirectFreeKickTaken));
    mmap.insert(pair<string, IEvent::EventType>("goal-kick", IEvent::GoalKick));
    mmap.insert(pair<string, IEvent::EventType>("goal-kick-taken", IEvent::GoalKickTaken));
    mmap.insert(pair<string, IEvent::EventType>("goal-opportunity", IEvent::GoalOpportunity));
    mmap.insert(pair<string, IEvent::EventType>("offside", IEvent::Offside));
    mmap.insert(pair<string, IEvent::EventType>("dribble", IEvent::Dribble));
    mmap.insert(pair<string, IEvent::EventType>("pass", IEvent::Pass));
    mmap.insert(pair<string, IEvent::EventType>("shoot", IEvent::Shoot));
    mmap.insert(pair<string, IEvent::EventType>("goal", IEvent::Goal));
    mmap.insert(pair<string, IEvent::EventType>("ball-dispute", IEvent::BallDispute));
    mmap.insert(pair<string, IEvent::EventType>("ball-kick", IEvent::BallKick));
    mmap.insert(pair<string, IEvent::EventType>("ball-hold", IEvent::BallHold));
    mmap.insert(pair<string, IEvent::EventType>("goalie-catch", IEvent::GoalieCatch));
    mmap.insert(pair<string, IEvent::EventType>("interception", IEvent::Interception));
    mmap.insert(pair<string, IEvent::EventType>("ball-move", IEvent::BallMove));
    mmap.insert(pair<string, IEvent::EventType>("player-move", IEvent::PlayerMove));
    mmap.insert(pair<string, IEvent::EventType>("possession-changed", IEvent::PossessionChanged));
    mmap.insert(pair<string, IEvent::EventType>("turnover", IEvent::Turnover));
    mmap.insert(pair<string, IEvent::EventType>("soccer-moment-changed", IEvent::SoccerMomentChanged));
    mmap.insert(pair<string, IEvent::EventType>("pass-chain", IEvent::PassChain));
    mmap.insert(pair<string, IEvent::EventType>("goal-pass-chain", IEvent::GoalPassChain));

    return mmap;
}

string eventType( const IEvent::EventType& event_type )
{
    static map<IEvent::EventType, string> event_type_to_str_map = createEventTypeToStringMap();

    return event_type_to_str_map.at(event_type);
}

IEvent::EventType eventType( const string& event_type_str )
{
    static map<string, IEvent::EventType> str_to_event_type_map = createStringToEventTypeMap();

    return str_to_event_type_map.at(event_type_str);
}



}
}
