#include "soccer_moment.h"

#include <soccer/events/ievent_visitor.h>

using namespace std;

namespace soccer {
namespace event {

map<SoccerMomentChanged::TeamSideMoment::Moment, string> SoccerMomentChanged::TeamSideMoment::createMomentToStringMap()
{
    map<Moment, string> mmap;
    mmap.insert(pair<Moment, string>(LOSING_POSSESSION, "losing_possession" ));
    mmap.insert(pair<Moment, string>(IN_POSSESSION, "in_possession" ));
    mmap.insert(pair<Moment, string>(NO_POSSESSION, "no_possession" ));
    mmap.insert(pair<Moment, string>(REGAINING_POSSESSION, "regaining_possession" ));

    return mmap;
}

map<string, SoccerMomentChanged::TeamSideMoment::Moment> SoccerMomentChanged::TeamSideMoment::createStringToMomentMap()
{
    map<string, Moment> mmap;
    mmap.insert(pair<string, Moment>("losing_possession", LOSING_POSSESSION ));
    mmap.insert(pair<string, Moment>("in_possession", IN_POSSESSION ));
    mmap.insert(pair<string, Moment>("no_possession", NO_POSSESSION ));
    mmap.insert(pair<string, Moment>("regaining_possession", REGAINING_POSSESSION ));

    return mmap;
}

SoccerMomentChanged::TeamSideMoment::Moment SoccerMomentChanged::TeamSideMoment::moment( const std::string& moment_str)
{
    static map<string, Moment> string_to_moment_map = createStringToMomentMap();
    return string_to_moment_map.at( moment_str );
}

const std::string& SoccerMomentChanged::TeamSideMoment::moment( Moment moment )
{
    map<Moment, string> moment_to_string_map = createMomentToStringMap();
    return moment_to_string_map.at( moment );
}


SoccerMomentChanged::TeamSideMoment::TeamSideMoment( TeamSide team_side, Moment moment )
    : M_team_side( team_side )
    , M_moment( moment )
{

}

SoccerMomentChanged::TeamSideMoment::TeamSideMoment( const TeamSideMoment& other)
    : M_team_side( other.M_team_side )
    , M_moment( other.M_moment )
{

}

TeamSide SoccerMomentChanged::TeamSideMoment::teamSide() const
{
    return M_team_side;
}

SoccerMomentChanged::TeamSideMoment::Moment SoccerMomentChanged::TeamSideMoment::moment() const
{
    return M_moment;
}

bool SoccerMomentChanged::TeamSideMoment::operator==( const TeamSideMoment& other)
{
    return ( M_team_side == other.M_team_side && M_moment == other.M_moment ) ||
           ( M_team_side == oppositeSide(other.M_team_side) &&
            ( ( M_moment == LOSING_POSSESSION && other.M_moment == REGAINING_POSSESSION ) ||
              ( M_moment == REGAINING_POSSESSION && other.M_moment == LOSING_POSSESSION )));
}

SoccerMomentChanged::SoccerMomentChanged(unsigned int time, TeamSideMoment team_moment)
    : Event( time )
    , M_team_moment( team_moment )
{

}

IEvent::EventType SoccerMomentChanged::type() const
{
    return IEvent::SoccerMomentChanged;
}

const SoccerMomentChanged::TeamSideMoment& SoccerMomentChanged::teamMoment() const
{
    return M_team_moment;
}

void SoccerMomentChanged::visit(IEventVisitor &visitor) const
{
    visitor.visit( *this );
}

}
}
