#ifndef XML_EVENT_PARSER_H
#define XML_EVENT_PARSER_H

#include <string>
#include <vector>

#include <stack>

#include <soccer/types.h>

#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/ContentHandler.hpp>

#include <tr1/memory>
#include <rcsc/geom/vector_2d.h>

namespace soccer {
namespace event {

class IEvent;

class XmlEventParser
        : protected xercesc::ContentHandler
{
public:
    XmlEventParser();
    ~XmlEventParser();

    void parse( const std::string& file_name );

    std::vector< std::tr1::shared_ptr<IEvent> > M_events;

    const std::string& source() const;
protected:

    // -----------------------------------------------------------------------
    //  Implementations of the SAX DocumentHandler interface
    // -----------------------------------------------------------------------
    void characters( const XMLCh* const chars,
                     const XMLSize_t length);

    void endDocument();

    void endElement ( const XMLCh* const uri,
                      const XMLCh* const localname,
                      const XMLCh* const qname);

    void ignorableWhitespace ( const XMLCh* const chars,
                               const XMLSize_t length );

    void processingInstruction ( const XMLCh* const target,
                                 const XMLCh* const data );

    void setDocumentLocator(const xercesc::Locator* const locator);

    void startDocument();

    void startElement ( const   XMLCh* const    uri,
                        const   XMLCh* const    localname,
                        const   XMLCh* const    qname,
                        const   xercesc::Attributes&     attrs );

    void startPrefixMapping( const XMLCh* const prefix,
                             const XMLCh* const uri );

    void endPrefixMapping( const XMLCh* const prefix);

    void skippedEntity( const XMLCh* const name );

private:
    bool parseEventBaseAttributes( const xercesc::Attributes& attrs,
                                   unsigned int& start_time,
                                   unsigned int& end_time );
    bool parseTeamSideAttribute( const xercesc::Attributes& attrs,
                                 const char* attr_name,
                                 soccer::TeamSide& side );

    bool parsePlayerNumberAttribute( const xercesc::Attributes& attrs,
                                     const char* attr_name,
                                     unsigned int& number);

    bool parseVector( const xercesc::Attributes& attrs,
                      rcsc::Vector2D &vector );

    xercesc::SAX2XMLReader* M_parser;
    std::stack< std::tr1::shared_ptr<IEvent> > M_stack;

    std::string M_source;
};

}
}

#endif // XML_EVENT_PARSER_H
