#include "types.h"

#include <cassert>

using namespace std;

namespace soccer {

    std::map<TeamSide, std::string> createTeamSideToStringMap()
    {
        map<TeamSide, std::string> map;
        map.insert(pair<TeamSide, std::string>( TS_LEFT, "left" ));
        map.insert(pair<TeamSide, std::string>( TS_RIGHT, "right"));
        map.insert(pair<TeamSide, std::string>( TS_UNKNOWN, "unknown"));

        return map;
    }

    const std::string& teamSide(TeamSide team_side)
    {
        static map<TeamSide, std::string> team_side_to_string = createTeamSideToStringMap();
        return team_side_to_string.at( team_side );
    }

    std::map<std::string, TeamSide> createStringToTeamSideMap()
    {
        map<std::string, TeamSide> map;
        map.insert(pair<std::string, TeamSide>( "left", TS_LEFT ));
        map.insert(pair<std::string, TeamSide>( "right", TS_RIGHT ));
        map.insert(pair<std::string, TeamSide>( "unknown", TS_UNKNOWN));

        return map;
    }

    TeamSide teamSide(const std::string& team_side)
    {
        static map<std::string, TeamSide> string_to_team_side = createStringToTeamSideMap();
        return string_to_team_side.at( team_side );
    }

    map<SoccerMomentAffiliation, std::string> createSoccerMomentToStringMap()
    {
        map<SoccerMomentAffiliation, std::string> map;
        map.insert(pair<SoccerMomentAffiliation, std::string>( SM_GAME_STOPPED_THEIR_POSSESION, "game_stopped_their_possession" ));
        map.insert(pair<SoccerMomentAffiliation, std::string>( SM_GAME_STOPPED_OUR_POSSESION, "game_stopped_our_possession" ));
        map.insert(pair<SoccerMomentAffiliation, std::string>( SM_OUR_POSSESION, "our_possession" ));
        map.insert(pair<SoccerMomentAffiliation, std::string>( SM_THEIR_POSSESION, "their_possession" ));
        map.insert(pair<SoccerMomentAffiliation, std::string>( SM_REGAINING_POSSESION, "regaining_possession" ));
        map.insert(pair<SoccerMomentAffiliation, std::string>( SM_LOSING_POSSESION, "losing_possession" ));

        return map;
    }

    const std::string& soccerMoment(SoccerMomentAffiliation soccer_moment)
    {
        static map<SoccerMomentAffiliation, std::string> soccer_moment_to_string = createSoccerMomentToStringMap();
        return soccer_moment_to_string.at( soccer_moment );
    }

    TeamSide oppositeSide(TeamSide side)
    {
        assert ( side != TS_UNKNOWN );
        return side == TS_LEFT ? TS_RIGHT : TS_LEFT;
    }

    TeamAffiliation oppositeAffiliation(TeamAffiliation affiliation)
    {
        assert ( affiliation != TA_UNKNOWN );
        return affiliation == TA_OUR ? TA_OPP : TA_OUR;
    }

}
