#ifndef SOCCER_TYPES_H
#define SOCCER_TYPES_H

#include <map>
#include <string>
#include <rcsc/geom/vector_2d.h>

namespace soccer {

    typedef unsigned int Unum;
    typedef std::map<Unum, rcsc::Vector2D> UnumPositions;

    enum TeamSide
    {
        TS_UNKNOWN,
        TS_LEFT,
        TS_RIGHT
    };

    enum TeamAffiliation
    {
        TA_UNKNOWN,
        TA_OUR,
        TA_OPP
    };

    enum TimePeriod
    {
        TP_FIRST_HALF,
        TP_SECOND_HALF,
        TP_EXTRA_TIME_FIRST_HALF,
        TP_EXTRA_TIME_SECOND_HALF
    };

    enum SoccerMomentAffiliation
    {
        SM_GAME_STOPPED_THEIR_POSSESION,
        SM_GAME_STOPPED_OUR_POSSESION,
        SM_OUR_POSSESION,
        SM_THEIR_POSSESION,
        SM_LOSING_POSSESION,
        SM_REGAINING_POSSESION
    };

    enum ScoreAssessment {
        ST_VERY_BAD,
        ST_BAD,
        ST_MEDIUM,
        ST_GOOD,
        ST_VERY_GOOD
    };

    std::map<TeamSide, std::string> createTeamSideToStringMap();
    const std::string& teamSide(TeamSide team_side);
    std::map<std::string, TeamSide> createStringToTeamSideMap();
    TeamSide teamSide(const std::string& team_side);
    std::map<SoccerMomentAffiliation, std::string> createSoccerMomentToStringMap();
    const std::string& soccerMoment(SoccerMomentAffiliation soccer_moment);
    TeamSide oppositeSide(TeamSide side);
    TeamAffiliation oppositeAffiliation(TeamAffiliation affiliation);


}
#endif // TYPES_H
