#ifndef FCPORTUGAL_STRATEGY_DPRE_H
#define FCPORTUGAL_STRATEGY_DPRE_H

#include <map>
#include <vector>

#include <soccer/types.h>

namespace soccer
{
    class IWorldModel;
	
    namespace strategy
    {

        class DPRE
        {
        public:
            // Mapping of real player nums to fullfilled player nums
            typedef std::map<soccer::Unum, soccer::Unum> Positionings;

            virtual ~DPRE() { }

            /*!
              \brief Get the positioning of a particular player by his number
              \return The player number whose role he is currently filling in
              */
            soccer::Unum positioning(soccer::Unum number) const;

            /*!
              \brief Check if it's helpful to exchange positionings
              \return True if al least one of the positionings changed
              */
            void update( );

            void reset( );

        protected:
            /*!
              \brief Retrieve the list of positionings that were changed in the last analysis
              */
            virtual void positioningsChanged(Positionings changed_positionings) = 0;

            /*!
            \brief Verify if player should be consider for DPRE
            \return True if the player is to be considered or false otherwise
             */
            virtual const std::vector<soccer::Unum>& eligiblePlayers( ) = 0;

            /*!
              \brief Value of exchanging the positionings of 2 players
              \return True if it is useful to exchange player positionings
              */
            virtual double exchangeValue(soccer::Unum number1,
                                         soccer::Unum number2) const = 0;

        protected:

            /// The current positionings for all players
            Positionings M_positionings;
        };

    }
}
#endif // FCPORTUGAL_STRATEGY_DPRE_H
