#ifndef FCP_STRATEGY_FLUX_MANAGER_H
#define FCP_STRATEGY_FLUX_MANAGER_H

#include <strategy/flux/flux.h>

namespace soccer
{
    class IWorldModel;

    namespace flux
    {


        class FluxManager
        {
        public:
            FluxManager();
            virtual ~FluxManager();

            void setFlux( const Flux *new_flux);

            virtual void update( soccer::IWorldModel *);

            rcsc::Vector2D fluxPosition() const;

        protected:
            /*!
              \brief Evaluate the usefulness of going from start_position to an end position
              \returns Returns a double precision percentage directly proportional to the usefulness
              */
            virtual double evaluate(const rcsc::Vector2D& start_position,
                                    const rcsc::Vector2D& end_position) = 0;


            virtual void fluxChanged( ) = 0;

            /*!
              \brief Get the value of the flux at a given position
              */
            double flux(const rcsc::Vector2D &position) const;

            const Flux* M_current_flux;
            rcsc::Vector2D M_previous_action_target_position, M_action_target_position;
            double M_previous_evaluation, M_action_evaluation, M_action_origin, M_action_previous_origin;
            rcsc::Vector2D M_next_position;     // Action Point Resulting from DMU
             // Action Value Resulting from DMU
            double M_flux_direction;		   // Flux Gradient Dir Resulting from DMU
            int M_dmu_action_out;	   // 3 - shoot, -3 own goal, -2 out
        };

    }
}

#endif // FLUX_MANAGER_H
