#ifndef FCPORTUGAL_SAMPLE_DATASET_H
#define FCPORTUGAL_SAMPLE_DATASET_H

#include <boost/shared_ptr.hpp>
#include <list>

namespace soccer
{
    namespace formation
    {

        template <typename SampleData>
        struct SampleDataSet {

            typedef boost::shared_ptr< SampleDataSet<SampleData> > Ptr; //!< shared pointer type.
            typedef boost::shared_ptr< const SampleDataSet<SampleData> > ConstPtr; //!< shared const pointer type.

            typedef std::list< SampleData > DataCont; //!< data container type.

            DataCont M_data_cont;

            /// Constructor
            SampleDataSet() : M_data_cont() { }

            /*!
              \brief clear all data.
             */
            void clear()
            {
                M_data_cont.clear();
            }

            /*!
              \brief get the data container.
              \return data container.
             */
            const DataCont dataCont() const
            {
                  return M_data_cont;
            }


            const SampleData* data( const size_t idx ) const
            {
                if ( M_data_cont.empty() || M_data_cont.size() < idx )
                {
                    return static_cast< const SampleData* >( NULL );
                }

                typename DataCont::const_iterator it = M_data_cont.begin();
                std::advance( it, idx );

                return &(*it);
            }

            /*!
             * Example line
             *  7 3
             *  9 40.99 -9.64
             *  10 38.16 1.29
             *  11 30.2 -15.93
             */

            bool read( std::istream& is)
            {
                // Check if it is EOL
                if (is.eof ())
                {
                    return true;
                }

                // Try to read player with ball samples data
                std::string line_buf;
                if ( ! std::getline( is, line_buf ) )
                {
                    std::cerr << " *** ERROR *** failed to read the number of sample data indexes changed."
                              << std::endl;
                    return false;
                }

                int num_samples = -1;
                if ( std::sscanf( line_buf.c_str(), " Begin Player With Ball Samples %u", &num_samples) != 1)
                {
                    std::cerr << " *** ERROR *** Illegal number of samples for Sample player data. "
                              << '[' << line_buf << ']'
                              << std::endl;
                    return false;
                }

                if ( num_samples == -1)
                {
                    return false;
                }

                for ( int i = 0 ; i < num_samples; i++)
                {
                    SampleData sample_data;
                    if (!sample_data.read(is))
                    {
                        return false;
                    }
                    M_data_cont.push_back(sample_data);
                }
                return true;
            }


            /*!
             * Example line
             *  7 3
             *  9 40.99 -9.64
             *  10 38.16 1.29
             *  11 30.2 -15.93
             */

        };
    }
}

#endif // FCPORTUGAL_SAMPLE_DATASET_H
