#include "formation_config.h"

using namespace boost::program_options;
using namespace std;

namespace soccer
{
    namespace strategy
    {

        const bool FormationConfig::USE_FORMATIONS = true;
        const std::string FormationConfig::FORMATIONS_DIR = "formations";
        const int FormationConfig::DEFAULT_FORMATION_ID = -1;
        const bool FormationConfig::SMOOTH_TRANSITIONS = false;
        const unsigned int FormationConfig::SMOOTH_TRANSITIONS_DELAY = 0u;

        FormationConfig::FormationConfig()
            : OptionsConfig<FormationConfig>("Formation Config")
        {
            // Formations options
            M_options_descriptions.add_options()
                ("help", "produce help message")
                ("use_formations",
                 value<bool>(&M_use_formations)->default_value(USE_FORMATIONS),
                 "Enable the use of formations")
                ("formations_dir",
                 value<string>(&M_formations_dir)->default_value(FORMATIONS_DIR),
                 "Directory that contains the definitions of the formations to use")
                ("formation_file",
                 value< vector<string> >(&M_formations_files)->composing(),
                 "Formation file (one or more names or paths of formation files to use")
                ("default_formation_id",
                 value<int>(&M_default_formation_id)->default_value(DEFAULT_FORMATION_ID),
                 "Identifier of the default formation to be used as a last resort")
                ("smooth_transitions",
                 value<bool>(&M_smooth_transitions)->default_value(SMOOTH_TRANSITIONS),
                 "Smooth player position changes between formations")
                ("smooth_transitions_delay",
                 value<unsigned int>(&M_smooth_transitions_delay)->default_value(SMOOTH_TRANSITIONS_DELAY),
                 "Time interval during which the positions to use in the current formation are adjusted based on the previous one")
            ;
        }


        bool FormationConfig::useFormations() const
        {
            return M_use_formations;
        }

        const std::string& FormationConfig::formationsDir() const
        {
            return M_formations_dir;
        }

        const std::vector<std::string>& FormationConfig::formationsFiles() const
        {
            return M_formations_files;
        }

        int FormationConfig::defaultFormationId() const
        {
            return M_default_formation_id;
        }


        bool FormationConfig::smoothTransitions() const
        {
            return M_smooth_transitions;
        }

        unsigned int FormationConfig::smoothTransitionsDelay() const
        {
            return M_smooth_transitions_delay;
        }

        void FormationConfig::print(std::ostream& os) const
        {
            os << "[Formation Config]" << endl;
            os << "use_formations: " << M_use_formations << endl;
            os << "formations_dir: " << M_formations_dir << endl;
            os << "default_formation_id: " << M_default_formation_id << endl;
            os << "smooth_transitions: " << M_smooth_transitions << endl;
            os << "smooth_transitions_delay" << M_smooth_transitions_delay << endl;

            os << "formation_file: ";
            for (vector<string>::const_iterator it = M_formations_files.begin();
                 it != M_formations_files.end(); it++)
            {
                os << *it << " ";
            }
            os << endl << endl;
         }
    }
}
