#include "league_config.h"

using namespace boost::program_options;
using namespace std;

namespace soccer
{
    namespace strategy
    {

        LeagueConfig::LeagueConfig()
            : OptionsConfig<LeagueConfig>("League Config")
        {
            // League options
            M_options_descriptions.add_options()
                ("help", "produce help message")
                ("field_length", value<double>(&M_field_length),
                 "Length of the soccer field")
                ("field_width", value<double>(&M_field_width),
                 "Width of the soccer field")
                ("penalty_area_length", value<double>(&M_penalty_area_length),
                 "Length of the penalty area")
                ("penalty_area_width", value<double>(&M_penalty_area_width),
                 "Width of the penalty area")
                ("goal_area_length", value<double>(&M_goal_area_length),
                 "Length of the goal area")
                ("goal_area_width", value<double>(&M_goal_area_width),
                 "Width of the goal area")
                ("goal_width", value<double>(&M_goal_width),
                 "Width of the goal")
                ("player_kick_distance_max", value<double>(&M_player_kick_distance_max),
                 "Maximum distance achieved by a player kick")
                ("Maximum speed that a player can reach", value<double>(&M_player_speed_max),
                 "Maximum speed a player can reach in standard units of time and distance")
                ("player_size", value<double>(&M_player_size),
                 "Size of the player agent")
                ("ball_size", value<double>(&M_ball_size),
                 "Size of the ball")
                ("ball_decay", value<double>(&M_ball_decay),
                 "Speed decay of the ball in free motion")
                ("ball_speed_max", value<double>(&M_ball_speed_max),
                 "Maximum speed that the ball can achieve after being kicked by a player")
                ("game_time", value<unsigned int>(&M_game_time),
                 "Duration of a full game")
                ("extra_time", value<unsigned int>(&M_extra_time),
                 "Duration of extra time")
                ("num_players_per_team", value<unsigned int>(&M_num_players_per_team),
                 "Number of players per team")
            ;
        }

        // Accessors
        double LeagueConfig::fieldLength() const
        {
            return M_field_length;
        }

        double LeagueConfig::fieldWidth() const
        {
            return M_field_width;
        }

        double LeagueConfig::penaltyAreaLength() const
        {
            return M_penalty_area_length;
        }

        double LeagueConfig::penaltyAreaWidth() const
        {
            return M_penalty_area_width;
        }


        double LeagueConfig::goalAreaLength() const
        {
            return M_goal_area_length;
        }

        double LeagueConfig::goalAreaWidth() const
        {
            return M_goal_area_width;
        }

        double LeagueConfig::goalWidth() const
        {
            return M_goal_width;
        }

        double LeagueConfig::playerKickDistanceMax() const
        {
            return M_player_kick_distance_max;
        }

        double LeagueConfig::playerSpeedMax() const
        {
            return M_player_speed_max;
        }

        double LeagueConfig::playerSize() const
        {
            return M_player_size;
        }

        unsigned int LeagueConfig::numPlayersPerTeam() const
        {
            return M_num_players_per_team;
        }

        double LeagueConfig::ballDecay() const
        {
            return M_ball_decay;
        }

        double LeagueConfig::ballSpeedMax() const
        {
            return M_ball_speed_max;
        }

        double LeagueConfig::ballSize() const
        {
            return M_ball_size;
        }

        unsigned int LeagueConfig::gameTime() const
        {
            return M_game_time;
        }
        unsigned int LeagueConfig::extraTime() const
        {
            return M_extra_time;
        }


        void LeagueConfig::print(std::ostream& os) const
        {
            os << "[League Config]" << endl;
            os << "field_length: " << M_field_length << endl;
            os << "field_width: " << M_field_width << endl;
            os << "penalty_area_length: " << M_penalty_area_length << endl;
            os << "penalty_area_width: " << M_penalty_area_width << endl;
            os << "goal_area_length: " << M_goal_area_length << endl;
            os << "goal_area_width: " << M_goal_area_width << endl;
            os << "goal_width: " << M_goal_width << endl;

            os << endl;
            os << "player_kick_distance_max: " << M_player_kick_distance_max << endl;
            os << "player_speed_max: " << M_player_speed_max << endl;
            os << "player_size: " << M_player_size << endl;

            os << endl;
            os << "ball_speed_max: " << M_ball_speed_max << endl;
            os << "ball_size: " << M_ball_size << endl;
            os << "ball_decay: " << M_ball_decay << endl;

            os << endl;
            os << "num_players_per_team: " << M_num_players_per_team << endl;
            os << "game_time: " << M_game_time << endl;
            os << "extra_time: " << M_extra_time << endl;

            os << endl;
         }

    }
}
