#ifndef FCPORTUGAL_TACTIC_H
#define FCPORTUGAL_TACTIC_H

#include <istream>
#include <map>

namespace soccer
{
    class IWorldModel;

    namespace tactic
    {
        class Tactic
        {
        public:
            typedef enum { TacticWithSituations } Type;

            Tactic();

            unsigned int id() const;
            unsigned int defaultFormation() const;
            unsigned int flux() const;
            unsigned int setplaysSet() const;
            virtual unsigned int numFormations() const;

            double weightFlux() const;
            double weightSafety() const;
            double weightEasy() const;
            double weightPass() const;
            double weightDribble() const;

            virtual bool usesGameSituations() const;

            virtual bool read(std::istream& is);

            // Pure virtual methods
            virtual Type type() const = 0;
        protected:
            unsigned int M_id;
            unsigned int M_default_formation; // Formation used, by default in each tactic
            unsigned int M_flux; // Flux used in the tactic => Shouldn't this be per formation also?
            unsigned int M_setplays_set; // Set of setplays used in the tactic

            // Weights
            double M_weight_flux; // Weight of the flux in the tactic
            double M_weight_safety; // Weight for safety in the tactic
            double M_weight_easy; // Weight for easy in the tactic => TODO Figure what this is for
            double M_weight_pass; // Weight for executing passes in the tactic
            double M_weight_dribble; // Weight for executing dribbles in the tactic
        };

        typedef std::map<unsigned int, Tactic* > Tactics;

    }
}

/*-------------------------------------------------------------------*/
/*!
  \brief operator '==' for fcportugal::Tactic
  \param lhs left hand side argument
  \param rhs right hand side argument
  \return boolean value
*/
inline
bool
operator==(  const soccer::tactic::Tactic& lhs,
             const soccer::tactic::Tactic& rhs )
{
    return lhs.id() == rhs.id();
}

/*-------------------------------------------------------------------*/
/*!
  \brief operator '!=' for fcportugal::Tactic
  \param lhs left hand side argument
  \param rhs right hand side argument
  \return boolean value
*/
inline
bool
operator!=(  const soccer::tactic::Tactic& lhs,
             const soccer::tactic::Tactic& rhs )
{
    return ! (lhs == rhs);
}

#endif // FCPORTUGAL_TACTIC_H
