#include "time_tactic.h"

#include <iostream>
#include <sstream>

#include <soccer/iworld_model.h>
#include <strategy/util.h>
#include <strategy/strategy.h>

using namespace soccer::strategy;
using namespace std;

namespace soccer
{
    namespace tactic
    {
        /// ScoreAssessmentTacticSelector
        ScoreAssessmentTacticSelector::ScoreAssessmentTacticSelector()
        {

        }

        void ScoreAssessmentTacticSelector::setDefaultTactic( unsigned int tactic_id)
        {
            M_default_tactic = tactic_id;
        }

        void ScoreAssessmentTacticSelector::addTactic( ScoreAssessment score_assessment,
                                                       unsigned int tactic_id )
        {
            M_tactics.insert(pair<ScoreAssessment, unsigned int>( score_assessment,
                                                                            tactic_id) );
        }

        unsigned int ScoreAssessmentTacticSelector::getTactic(const IWorldModel *world_model) const
        {
            ScoreAssessmentTactics::const_iterator itt = M_tactics.find( world_model->scoreAssessment() );
            if ( itt == M_tactics.end() )
            {
                return M_default_tactic;
            }
            return itt->second;
        }


        /// TimeTacticSelector
        unsigned int TimeTacticSelector::S_any_opponent = 0;

        TimeTacticSelector::TimeTacticSelector( )
        {

        }

        bool TimeTacticSelector::read(std::istream& is,
                                      unsigned int num_opponent_tactics )
        {
            std::istringstream istr(util::getNextLine(is));
            unsigned int start_time = 0, end_time = 0;
            istr >> start_time;
            istr >> end_time;

            for (unsigned int o = 0; o < num_opponent_tactics; o++)
            {
                unsigned int opp = 0;
                istr >> opp;

                for(int re = 0 ; re < 5; re++)
                {
                    // HACK 5 values for enum ScoreAssessment
                    unsigned int tactic = 0;
                    istr >> tactic;
                    this->addTactic( opp,
                                     (ScoreAssessment) re,
                                     start_time,
                                     end_time,
                                     tactic);
                }
            }
            return true;
        }


        unsigned int TimeTacticSelector::getTactic( const IWorldModel* world_model ) const
        {
            // TODO Determine current opponent
            ScoreAssessmentTactics::const_iterator itsat = M_default_tactics.find( world_model->scoreAssessment() );
            if ( itsat == M_default_tactics.end() )
            {
                // Return default tactic
                assert(false);
                return 0;
            }

            const TimeTactics& time_tactics = itsat->second;
            unsigned int current_time = world_model->currentTime();
            for ( TimeTactics::const_iterator ittt = time_tactics.begin();
                  ittt != time_tactics.end() ; ittt++ )
            {
                const TimeTactic& time_tactic = *ittt;
                if ( time_tactic.start_time <= current_time &&
                     time_tactic.end_time >= current_time )
                {
                    return time_tactic.tactic_id;
                }
            }

            // Return default tactic
            assert(false);
            return 0;
        }


        void TimeTacticSelector::addTactic( ScoreAssessment score_assessment,
                                            unsigned int start_time,
                                            unsigned int end_time,
                                            unsigned int tactic_id)
        {
            TimeTactic time_tactic;
            time_tactic.start_time = start_time;
            time_tactic.end_time = end_time;
            time_tactic.tactic_id = tactic_id;

            ScoreAssessmentTactics::iterator itsat = M_default_tactics.find(score_assessment);
            if ( itsat == M_default_tactics.end() )
            {
                TimeTactics time_tactics;
                time_tactics.push_back(time_tactic);
                M_default_tactics.insert(std::pair<ScoreAssessment, TimeTactics>( score_assessment, time_tactics ));
                return;
            }

            TimeTactics& time_tactics = itsat->second;
            time_tactics.push_back(time_tactic);
        }

        void TimeTacticSelector::addTactic( unsigned int opponent,
                                            ScoreAssessment score_assessment,
                                            unsigned int start_time,
                                            unsigned int end_time,
                                            unsigned int tactic_id)
        {
            if ( opponent == TimeTacticSelector::S_any_opponent )
            {
                addTactic( score_assessment, start_time, end_time, tactic_id);
                return;
            }

            OpponentScoreAssessmentTactics::iterator ito = M_opponent_tactics.find( opponent );

            TimeTactic time_tactic;
            time_tactic.start_time = start_time;
            time_tactic.end_time = end_time;
            time_tactic.tactic_id = tactic_id;

            if ( ito == M_opponent_tactics.end() )
            {
                ScoreAssessmentTactics score_assessment_tactics;
                TimeTactics time_tactics;
                time_tactics.push_back(time_tactic);
                score_assessment_tactics.insert(pair<ScoreAssessment,
                                                     TimeTactics>( score_assessment,
                                                                    time_tactics));
                M_opponent_tactics.insert(pair<unsigned int, ScoreAssessmentTactics>( opponent,
                                                                                      score_assessment_tactics ));
                return;
            }

            ScoreAssessmentTactics& score_assessment_tactics = ito->second;
            ScoreAssessmentTactics::iterator its = score_assessment_tactics.find( score_assessment );
            if ( its == score_assessment_tactics.end() )
            {
                TimeTactics time_tactics;
                time_tactics.push_back(time_tactic);
                score_assessment_tactics.insert(pair< ScoreAssessment,
                                                      TimeTactics>( score_assessment,
                                                                    time_tactics));
                return;
            }

            TimeTactics& time_tactics = its->second;
            time_tactics.push_back( time_tactic );
        }

    }
}
